@@ -1,4 +1,3 @@
-Midnight Commander (c) 1995-1999, Free Software Foundation
 Tip: C-x t vlo na pkazovou dku oznaen soubory.
 Tip: C-x p vlo na pkazovou dku jmno aktulnho adrese.
 Tip: K doplovn pouijte M-Tab (nebo Esc+Tab). Dvojitm zadn zskte seznam.
@@ -7,9 +6,8 @@ Tip: Chcete zadat speciln znak(nap. tabeltor)? Pouijte Ctrl-Q a dan znak.
 Tip: Nechcete ji vidt tipy? Vypnte je v Nastaven/Zobrazen/Dal nastaven.
 Tip: Adrese vyberete pidnm lomtka na konec vzorku.
 Tip: Pokud V terminl nezn nebo nem funkn klvesy, pouijte Esc+slo.
-Tip: Domc strnka programu Midnight Commander je na http://www.gnome.org/mc/
-Tip: Posledn verze programu Midnight Commander je na ftp://ftp.gnome.org/pub/GNOME/sources/mc
-Tip: Zprvy o chybch poslejte na mc-bugs@nuclecu.unam.mx (anglicky).
+Tip: Domc strnka programu Midnight Commander je http://www.midnight-commander.org/
+Tip: Zprvy o chybch poslejte na mc-devel@gnome.org (anglicky).
 Tip: Pomoc klvesy Tab zmnte aktuln panel.
 Tip: Stiskem Enter na souboru .tar nebo .tar.gz si prohldnete obsah.
 Tip: Existuje tak manulov strnka [mc(1)].
@@ -22,17 +20,15 @@ Tip: Chcete opt v shell? Stisknte C-o. Do MC se vrtte opt pes C-o.
 Tip: Nastavenm promnn CDPATH mete etit klvesnici.
 Tip: Pokud chcete vidt soubory zanajc tekou, eknte to v Nastaven.
 Tip: Chcete vidt zlohy svch soubor (kon ~). Pouijte Nastaven.
-Tip: Mte opravdu mnoho pamti? Zkuste nastavit limit ve VFS na -1.
 Tip: Doplovn funguje ve vstupnch dkch vech dialog. Pouijte M-Tab.
 Tip: Na terminlech s pomalou odezvou skuste na parametr -s (mc -s).
-Tip: S nalezenmi soubory mete pracovat. Stiknte tlatko Panelizace.
+Tip: S nalezenmi soubory mete pracovat. Stisknte tlatko Panelizace.
 Tip: Pro opravdu sloit operace zkuste pkaz Pkaz/Extern panelizace.
 Tip: Pokud chcete zmnit adres pi prci na pkoazov dce, zkuste M-c.
 Poznmka: Pkazy shellu nefunguj, pokud nepracujete na lokln systmu soubor.
 Tip: Smazan text obnovte C-y.
-Tip: Pokud nkter klvesy nefunfuj, pouijte menu Nastaven/uen klves.
+Tip: Pokud nkter klvesy nefunguj, pouijte menu Nastaven/uen klves.
 Tip: F13 (nebo Shift-F3) vyvol prohlen bez zpracovn obsahu souboru.
-Tip: Pokud v xtermu nepracuj klvesy, zkuste soubory xterm.ad, .ti a .tcap.
 Tip: Editor pro klvesu F4 urte pomoc promnn prosted EDITOR.
 Tip: Extern prohle urte promnnou PAGER.
 Tip: Vechny dosti o potvrzen mete zakzat v Nastaven/Potvrzovn.
