@@ -1,6 +1,7 @@
 /* Some misc dialog boxes for the program.
    
-   Copyright (C) 1994, 1995 the Free Software Foundation
+   Copyright (C) 1994, 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+   2005, 2006 Free Software Foundation, Inc.
    
    Authors: 1994, 1995 Miguel de Icaza
             1995 Jakub Jelinek
@@ -17,23 +18,25 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
-#include <string.h>
+
+#include <ctype.h>
+#include <signal.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <signal.h>
-#include <ctype.h>
 
 #include "global.h"
 #include "tty.h"
 #include "win.h"		/* Our window tools */
 #include "color.h"		/* Color definitions */
-#include "dlg.h"		/* The nice dialog manager */
+#include "dialog.h"		/* The nice dialog manager */
 #include "widget.h"		/* The widgets for the nice dialog manager */
-#include "dialog.h"		/* For do_refresh() */
 #include "wtools.h"
 #include "setup.h"		/* For profile_name */
 #include "profile.h"		/* Load/save user formats */
@@ -45,18 +48,21 @@
 #include "main.h"		/* For the confirm_* variables */
 #include "tree.h"
 #include "layout.h"		/* for get_nth_panel_name proto */
-#include "background.h"
+#include "background.h"		/* task_list */
 
 #ifdef HAVE_CHARSET
 #include "charsets.h"
 #include "selcodepage.h"
 #endif
 
-#include "../vfs/vfs.h"		/* vfs_timeout */
 #ifdef USE_NETCODE
 #   include "../vfs/ftpfs.h"
 #endif
 
+#ifdef USE_VFS
+#include "../vfs/gc.h"
+#endif
+
 static int DISPLAY_X = 45, DISPLAY_Y = 14;
 
 static Dlg_head *dd;
@@ -69,7 +75,7 @@ static int current_mode;
 static char **displays_status;
 
 /* Controls whether the array strings have been translated */
-static char *displays [LIST_TYPES] = {
+static const char *displays [LIST_TYPES] = {
     N_("&Full file list"),
     N_("&Brief file list"),
     N_("&Long file list"),
@@ -81,62 +87,61 @@ static char *displays [LIST_TYPES] = {
 
 static int user_hotkey = 'u';
 
-static int
-display_callback (struct Dlg_head *h, int id, int Msg)
+static cb_ret_t
+display_callback (struct Dlg_head *h, dlg_msg_t msg, int parm)
 {
-    switch (Msg){
-    case DLG_DRAW:
-	common_dialog_repaint (h);
-	break;
-
+    switch (msg) {
     case DLG_UNFOCUS:
-	if((WRadio *) h->current->widget == my_radio){
-	    assign_text (status, displays_status [my_radio->sel]);
+	if (dlg_widget_active (my_radio)) {
+	    assign_text (status, displays_status[my_radio->sel]);
 	    input_set_point (status, 0);
 	}
-	break;
-	
+	return MSG_HANDLED;
+
     case DLG_KEY:
-	if (id == '\n'){
-	    if((WRadio *) h->current->widget == my_radio){
-		assign_text (status, displays_status [my_radio->sel]);
+	if (parm == '\n') {
+	    if (dlg_widget_active (my_radio)) {
+		assign_text (status, displays_status[my_radio->sel]);
 		dlg_stop (h);
-		break;
+		return MSG_HANDLED;
 	    }
-	    
-	    if ((WInput *) h->current->widget == user){
+
+	    if (dlg_widget_active (user)) {
 		h->ret_value = B_USER + 6;
 		dlg_stop (h);
-		break;
+		return MSG_HANDLED;
 	    }
-	
-	    if ((WInput *) h->current->widget == status){
+
+	    if (dlg_widget_active (status)) {
 		h->ret_value = B_USER + 7;
 		dlg_stop (h);
-		break;
+		return MSG_HANDLED;
 	    }
 	}
 
-	if (tolower(id) == user_hotkey && h->current->widget != (Widget *) user
-	    && h->current->widget != (Widget *) status){
+	if (tolower (parm) == user_hotkey && dlg_widget_active (user)
+	    && dlg_widget_active (status)) {
 	    my_radio->sel = 3;
-	    dlg_select_widget (h, my_radio); /* force redraw */
-	    dlg_select_widget (h, user);
+	    dlg_select_widget (my_radio);	/* force redraw */
+	    dlg_select_widget (user);
 	    return MSG_HANDLED;
 	}
+	return MSG_NOT_HANDLED;
+
+    default:
+	return default_dlg_callback (h, msg, parm);
     }
-    return MSG_NOT_HANDLED;
 }
 
 static void
 display_init (int radio_sel, char *init_text, int _check_status,
 	      char **_status)
 {
-    static char *display_title = N_("Listing mode");
+    static const char *display_title = N_("Listing mode");
     static int i18n_displays_flag;
-    char *user_mini_status = _("user &Mini status");
-    char *ok_button = _("&OK");
-    char *cancel_button = _("&Cancel");
+    const char *user_mini_status = _("user &Mini status");
+    const char *ok_button = _("&OK");
+    const char *cancel_button = _("&Cancel");
 
     static int button_start = 30;
 
@@ -144,7 +149,7 @@ display_init (int radio_sel, char *init_text, int _check_status,
 
     if (!i18n_displays_flag) {
 	int i, l, maxlen = 0;
-	char *cp;
+	const char *cp;
 
 	display_title = _(display_title);
 	for (i = 0; i < LIST_TYPES; i++) {
@@ -174,39 +179,39 @@ display_init (int radio_sel, char *init_text, int _check_status,
 	/* get hotkey of user-defined format string */
 	cp = strchr (displays[USER_TYPE], '&');
 	if (cp != NULL && *++cp != '\0')
-	    user_hotkey = tolower (*cp);
+	    user_hotkey = tolower ((unsigned char) *cp);
 
 	i18n_displays_flag = 1;
     }
     dd = create_dlg (0, 0, DISPLAY_Y, DISPLAY_X, dialog_colors,
 		     display_callback, "[Listing Mode...]", display_title,
-		     DLG_CENTER);
+		     DLG_CENTER | DLG_REVERSE);
 
     add_widget (dd,
 		button_new (4, button_start, B_CANCEL, NORMAL_BUTTON,
-			    cancel_button, 0, 0, "cancel-button"));
+			    cancel_button, 0));
 
     add_widget (dd,
 		button_new (3, button_start, B_ENTER, DEFPUSH_BUTTON,
-			    ok_button, 0, 0, "ok-button"));
+			    ok_button, 0));
 
     status =
 	input_new (10, 9, INPUT_COLOR, DISPLAY_X - 14, _status[radio_sel],
-		   "mini-input");
+		   "mini-input", INPUT_COMPLETE_DEFAULT);
     add_widget (dd, status);
     input_set_point (status, 0);
 
     check_status =
-	check_new (9, 5, _check_status, user_mini_status, "mini-status");
+	check_new (9, 5, _check_status, user_mini_status);
     add_widget (dd, check_status);
 
     user =
 	input_new (7, 9, INPUT_COLOR, DISPLAY_X - 14, init_text,
-		   "user-fmt-input");
+		   "user-fmt-input", INPUT_COMPLETE_DEFAULT);
     add_widget (dd, user);
     input_set_point (user, 0);
 
-    my_radio = radio_new (3, 5, LIST_TYPES, displays, 1, "radio");
+    my_radio = radio_new (3, 5, LIST_TYPES, displays);
     my_radio->sel = my_radio->pos = current_mode;
     add_widget (dd, my_radio);
 }
@@ -216,7 +221,7 @@ display_box (WPanel *panel, char **userp, char **minip, int *use_msformat, int n
 {
     int result, i;
     char *section = NULL;
-    char *p;
+    const char *p;
 
     if (!panel) {
         p = get_nth_panel_name (num);
@@ -226,7 +231,7 @@ display_box (WPanel *panel, char **userp, char **minip, int *use_msformat, int n
         panel->user_mini_status = 0;
 	for (i = 0; i < LIST_TYPES; i++)
     	    panel->user_status_format[i] = g_strdup (DEFAULT_USER_FORMAT);
-        section = g_strconcat ("Temporal:", p, NULL);
+        section = g_strconcat ("Temporal:", p, (char *) NULL);
         if (!profile_has_section (section, profile_name)) {
             g_free (section);
             section = g_strdup (p);
@@ -261,97 +266,99 @@ display_box (WPanel *panel, char **userp, char **minip, int *use_msformat, int n
     return result;
 }
 
-static int SORT_X = 40, SORT_Y = 14;
+static int SORT_X = 60, SORT_Y = 14;
 
-static char *sort_orders_names [SORT_TYPES];
+static const char *sort_orders_names [SORT_TYPES];
 
 sortfn *
-sort_box (sortfn *sort_fn, int *reverse, int *case_sensitive)
+sort_box (sortfn *sort_fn, int *reverse, int *case_sensitive, int *exec_first)
 {
     int i, r, l;
     sortfn *result;
-    WCheck *c, *case_sense;
+    WCheck *c, *case_sense, *exec_ff;
 
-	char* ok_button = _("&OK");
-	char* cancel_button = _("&Cancel");
-	char* reverse_label = _("&Reverse");
-	char* case_label = _("case sensi&tive");
-	char* sort_title = _("Sort order");
+    const char *ok_button = _("&OK");
+    const char *cancel_button = _("&Cancel");
+    const char *reverse_label = _("&Reverse");
+    const char *case_label = _("case sensi&tive");
+    const char *sort_title = _("Sort order");
+    const char *exec_label = _("Executable first");
 
-	static int i18n_sort_flag = 0, check_pos = 0, button_pos = 0;
+    static int i18n_sort_flag = 0, check_pos = 0, button_pos = 0;
 
-	if (!i18n_sort_flag)
-	{
-		int maxlen = 0;
-		for (i = SORT_TYPES-1; i >= 0; i--)
-		{
-			sort_orders_names [i] = _(sort_orders [i].sort_name);
-			r = strlen (sort_orders_names [i]);
-			if (r > maxlen)
-				maxlen = r;
-		}
-
-		check_pos = maxlen + 9;
-
-		r = strlen (reverse_label) + 4;
-		i = strlen (case_label) + 4;
-		if (i > r)
-			r = i;
-		
-		l = strlen (ok_button) + 6;
-		i = strlen (cancel_button) + 4;
-		if (i > l)
-			l = i;
-			
-		i = check_pos + max(r,l) + 2;
-
-		if (i > SORT_X)
-			SORT_X = i;
-
-		i = strlen (sort_title) + 6;
-		if (i > SORT_X)
-			SORT_X = i;
-
-		button_pos = SORT_X - l - 2;
-
-		i18n_sort_flag = 1;
+    if (!i18n_sort_flag) {
+	int maxlen = 0;
+	for (i = SORT_TYPES - 1; i >= 0; i--) {
+	    sort_orders_names[i] = _(sort_orders[i].sort_name);
+	    r = strlen (sort_orders_names[i]);
+	    if (r > maxlen)
+		maxlen = r;
 	}
 
+	check_pos = maxlen + 9;
+
+	r = strlen (reverse_label) + 4;
+	i = strlen (case_label) + 4;
+	if (i > r)
+	    r = i;
+
+	l = strlen (ok_button) + 6;
+	i = strlen (cancel_button) + 4;
+	if (i > l)
+	    l = i;
+
+	i = check_pos + max (r, l) + 2;
+
+	if (i > SORT_X)
+	    SORT_X = i;
+
+	i = strlen (sort_title) + 6;
+	if (i > SORT_X)
+	    SORT_X = i;
+
+	button_pos = SORT_X - l - 2;
+
+	i18n_sort_flag = 1;
+    }
+
     result = 0;
-    
+
     for (i = 0; i < SORT_TYPES; i++)
-	if ((sortfn *) (sort_orders [i].sort_fn) == sort_fn){
+	if ((sortfn *) (sort_orders[i].sort_fn) == sort_fn) {
 	    current_mode = i;
 	    break;
 	}
-    
+
     dd = create_dlg (0, 0, SORT_Y, SORT_X, dialog_colors, NULL,
-		     "[Sort Order...]", sort_title, DLG_CENTER);
+		     "[Sort Order...]", sort_title, DLG_CENTER | DLG_REVERSE);
 
-    add_widget (dd, 
-		button_new (10, button_pos, B_CANCEL, NORMAL_BUTTON, cancel_button, 
-		0, 0, "cancel-button"));
+    add_widget (dd,
+		button_new (10, button_pos, B_CANCEL, NORMAL_BUTTON,
+			    cancel_button, 0));
 
-    add_widget (dd, 
-		button_new (9, button_pos, B_ENTER, DEFPUSH_BUTTON, ok_button,
-		0, 0, "ok-button"));
+    add_widget (dd,
+		button_new (9, button_pos, B_ENTER, DEFPUSH_BUTTON,
+			    ok_button, 0));
 
-    case_sense = check_new (4, check_pos, *case_sensitive, case_label, "case-check");
+    exec_ff = check_new (5, check_pos, *exec_first, exec_label);
+    add_widget (dd, exec_ff);
+    case_sense = check_new (4, check_pos, *case_sensitive, case_label);
     add_widget (dd, case_sense);
-    c = check_new (3, check_pos, *reverse, reverse_label, "reverse-check");
+    c = check_new (3, check_pos, *reverse, reverse_label);
     add_widget (dd, c);
 
-    my_radio = radio_new (3, 3, SORT_TYPES, sort_orders_names, 1, "radio-1");
+    my_radio = radio_new (3, 3, SORT_TYPES, sort_orders_names);
     my_radio->sel = my_radio->pos = current_mode;
-    
+
     add_widget (dd, my_radio);
     run_dlg (dd);
 
     r = dd->ret_value;
-    if (r != B_CANCEL){
-	result = (sortfn *) sort_orders [my_radio->sel].sort_fn;
+    if (r != B_CANCEL) {
+	result = (sortfn *) sort_orders[my_radio->sel].sort_fn;
 	*reverse = c->state & C_BOOL;
 	*case_sensitive = case_sense->state & C_BOOL;
+	*exec_first = exec_ff->state & C_BOOL;
     } else
 	result = sort_fn;
     destroy_dlg (dd);
@@ -359,29 +366,32 @@ sort_box (sortfn *sort_fn, int *reverse, int *case_sensitive)
     return result;
 }
 
-#define CONFY 10
+#define CONFY 11
 #define CONFX 46
 
 static int my_delete;
+static int my_directory_hotlist_delete;
 static int my_overwrite;
 static int my_execute;
 static int my_exit;
 
 static QuickWidget conf_widgets [] = {
 { quick_button,   4, 6, 4, CONFY, N_("&Cancel"),
-      0, B_CANCEL, 0, 0, "c" },
+      0, B_CANCEL, 0, 0, NULL },
 { quick_button,   4, 6, 3, CONFY, N_("&OK"),
-      0, B_ENTER, 0, 0, "o" },
+      0, B_ENTER, 0, 0, NULL },
 
+{ quick_checkbox, 1, 13, 7, CONFY, N_(" confirm di&Rectory hotlist delete "),
+      11, 0, &my_directory_hotlist_delete, NULL, NULL },
 { quick_checkbox, 1, 13, 6, CONFY, N_(" confirm &Exit "),
-      9, 0, &my_exit, 0, "e" },
+      9, 0, &my_exit, 0, NULL },
 { quick_checkbox, 1, 13, 5, CONFY, N_(" confirm e&Xecute "),
-      10, 0, &my_execute, 0, "x" },
+      10, 0, &my_execute, 0, NULL },
 { quick_checkbox, 1, 13, 4, CONFY, N_(" confirm o&Verwrite "),
-      10, 0, &my_overwrite, 0, "ov" },
+      10, 0, &my_overwrite, 0, NULL },
 { quick_checkbox, 1, 13, 3, CONFY, N_(" confirm &Delete "),
-      9, 0, &my_delete, 0, "de" },
-{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
+      9, 0, &my_delete, 0, NULL },
+NULL_QuickWidget
 };
 
 static QuickDialog confirmation =
@@ -438,12 +448,14 @@ confirm_box (void)
     my_overwrite = confirm_overwrite;
     my_execute   = confirm_execute;
     my_exit      = confirm_exit;
+    my_directory_hotlist_delete = confirm_directory_hotlist_delete;
 
     if (quick_dialog (&confirmation) != B_CANCEL){
 	confirm_delete    = my_delete;
 	confirm_overwrite = my_overwrite;
 	confirm_execute   = my_execute;
 	confirm_exit      = my_exit;
+	confirm_directory_hotlist_delete = my_directory_hotlist_delete;
     }
 }
 
@@ -456,19 +468,19 @@ confirm_box (void)
 static int new_mode;
 static int new_meta;
 
-static char *display_bits_str [] =
+static const char *display_bits_str [] =
 { N_("Full 8 bits output"), N_("ISO 8859-1"), N_("7 bits") };
 
 static QuickWidget display_widgets [] = {
 { quick_button,   4,  6,    4, DISPY, N_("&Cancel"),
-      0, B_CANCEL, 0, 0, "c" },
+      0, B_CANCEL, 0, 0, NULL },
 { quick_button,   4,  6,    3, DISPY, N_("&OK"),
-      0, B_ENTER, 0, 0, "o" },
+      0, B_ENTER, 0, 0, NULL },
 { quick_checkbox, 4, DISPX, 7, DISPY, N_("F&ull 8 bits input"),
-      0, 0, &new_meta, 0, "u" },
+      0, 0, &new_meta, 0, NULL },
 { quick_radio,    4, DISPX, 3, DISPY, "", 3, 0,
-      &new_mode, display_bits_str, "r" },
-{ 0,              0, 0, 0, 0, 0,  0, 0, 0, 0, 0 }
+      &new_mode, const_cast(char **, display_bits_str), NULL },
+NULL_QuickWidget
 };
 
 static QuickDialog display_bits =
@@ -548,54 +560,58 @@ static int new_display_codepage;
 static WLabel *cplabel;
 static WCheck *inpcheck;
 
-static int sel_charset_button( int action, void *param )
+static int
+sel_charset_button (int action)
 {
-    char *cpname, buf[64];
-    new_display_codepage = select_charset( new_display_codepage, 1 );
+    const char *cpname;
+    char buf[64];
+    new_display_codepage = select_charset (new_display_codepage, 1);
     cpname = (new_display_codepage < 0)
-	     ? _("Other 8 bit")
-	     : codepages[ new_display_codepage ].name;
-    g_snprintf( buf, sizeof (buf), "%-27s", cpname ); /* avoid strange bug with label repainting */
-    label_set_text( cplabel, buf );
+	? _("Other 8 bit")
+	: codepages[new_display_codepage].name;
+
+    /* avoid strange bug with label repainting */
+    g_snprintf (buf, sizeof (buf), "%-27s", cpname);
+    label_set_text (cplabel, buf);
     return 0;
 }
 
 static Dlg_head *
 init_disp_bits_box (void)
 {
-    char *cpname;
+    const char *cpname;
     Dlg_head *dbits_dlg;
 
-    do_refresh();
+    do_refresh ();
 
-    dbits_dlg = create_dlg( 0, 0, DISPY, DISPX, dialog_colors,
-		NULL, "[Display bits]", _(" Display bits "), DLG_CENTER);
+    dbits_dlg =
+	create_dlg (0, 0, DISPY, DISPX, dialog_colors, NULL,
+		    "[Display bits]", _(" Display bits "), DLG_CENTER | DLG_REVERSE);
 
-    add_widget( dbits_dlg,
-		label_new( 3, 4, _("Input / display codepage:"), NULL));
+    add_widget (dbits_dlg,
+		label_new (3, 4, _("Input / display codepage:")));
 
     cpname = (new_display_codepage < 0)
-	     ? _("Other 8 bit")
-	     : codepages[ new_display_codepage ].name;
-    cplabel = label_new( 4, 4, cpname, NULL);
-    add_widget( dbits_dlg, cplabel );
-	
-    add_widget( dbits_dlg,
-		button_new( DISPY - 3, DISPX / 2 + 3, B_CANCEL,
-			    NORMAL_BUTTON, _("&Cancel"), 0, 0, NULL ) );
-    add_widget( dbits_dlg,
-		button_new( DISPY - 3, 7, B_ENTER,
-			    NORMAL_BUTTON, _("&OK"), 0, 0, NULL ) );
+	? _("Other 8 bit")
+	: codepages[new_display_codepage].name;
+    cplabel = label_new (4, 4, cpname);
+    add_widget (dbits_dlg, cplabel);
 
-    inpcheck = check_new( 6, 4, !use_8th_bit_as_meta,
-			  _("F&ull 8 bits input"), NULL );
-    add_widget( dbits_dlg, inpcheck );
+    add_widget (dbits_dlg,
+		button_new (DISPY - 3, DISPX / 2 + 3, B_CANCEL,
+			    NORMAL_BUTTON, _("&Cancel"), 0));
+    add_widget (dbits_dlg,
+		button_new (DISPY - 3, 7, B_ENTER, NORMAL_BUTTON, _("&OK"),
+			    0));
+
+    inpcheck =
+	check_new (6, 4, !use_8th_bit_as_meta, _("F&ull 8 bits input"));
+    add_widget (dbits_dlg, inpcheck);
 
     cpname = _("&Select");
-    add_widget( dbits_dlg,
-		button_new( 4, DISPX - 8 - strlen(cpname) , B_USER,
-			    NORMAL_BUTTON, cpname,
-			    sel_charset_button, 0, NULL ) );
+    add_widget (dbits_dlg,
+		button_new (4, DISPX - 8 - strlen (cpname), B_USER,
+			    NORMAL_BUTTON, cpname, sel_charset_button));
 
     return dbits_dlg;
 }
@@ -612,21 +628,22 @@ display_bits_box (void)
     run_dlg (dbits_dlg);
 
     if (dbits_dlg->ret_value == B_ENTER) {
-	char *errmsg;
+	const char *errmsg;
 	display_codepage = new_display_codepage;
-	errmsg = init_translation_table( source_codepage, display_codepage );
+	errmsg =
+	    init_translation_table (source_codepage, display_codepage);
 	if (errmsg)
-	    message( 1, MSG_ERROR, "%s", errmsg );
+	    message (1, MSG_ERROR, "%s", errmsg);
 #ifndef HAVE_SLANG
-	meta( stdscr, display_codepage != 0 );
+	meta (stdscr, display_codepage != 0);
 #else
-	SLsmg_Display_Eight_Bit
-	    = (display_codepage != 0 && display_codepage != 1) ? 128 : 160;
+	SLsmg_Display_Eight_Bit = (display_codepage != 0
+				   && display_codepage != 1) ? 128 : 160;
 #endif
-	use_8th_bit_as_meta = ! (inpcheck->state & C_BOOL);
-    }    
-    destroy_dlg( dbits_dlg );
-    repaint_screen();
+	use_8th_bit_as_meta = !(inpcheck->state & C_BOOL);
+    }
+    destroy_dlg (dbits_dlg);
+    repaint_screen ();
 }
 
 #endif /* HAVE_CHARSET */
@@ -635,29 +652,27 @@ display_bits_box (void)
 #define TREE_Y 20
 #define TREE_X 60
 
-static int
-tree_callback (struct Dlg_head *h, int id, int msg)
+static cb_ret_t
+tree_callback (struct Dlg_head *h, dlg_msg_t msg, int parm)
 {
-    switch (msg){
+    switch (msg) {
 
     case DLG_POST_KEY:
 	/* The enter key will be processed by the tree widget */
-	if (id == '\n') {
+	if (parm == '\n') {
 	    h->ret_value = B_ENTER;
 	    dlg_stop (h);
 	}
 	return MSG_HANDLED;
 	
-    case DLG_DRAW:
-	common_dialog_repaint (h);
-	break;
+    default:
+	return default_dlg_callback (h, msg, parm);
     }
-    return MSG_NOT_HANDLED;
 }
 
 /* Show tree in a box, not on a panel */
 char *
-tree_box (char *current_dir)
+tree_box (const char *current_dir)
 {
     WTree    *mytree;
     Dlg_head *dlg;
@@ -666,13 +681,13 @@ tree_box (char *current_dir)
 
     /* Create the components */
     dlg = create_dlg (0, 0, TREE_Y, TREE_X, dialog_colors,
-		      tree_callback, "[Directory Tree]", NULL, DLG_CENTER);
+		      tree_callback, "[Directory Tree]", NULL, DLG_CENTER | DLG_REVERSE);
     mytree = tree_new (0, 2, 2, TREE_Y - 6, TREE_X - 5);
     add_widget (dlg, mytree);
     bar = buttonbar_new(1);
     add_widget (dlg, bar);
-    bar->widget.x = 0;
-    bar->widget.y = LINES - 1;
+    ((Widget *) bar)->x = 0;
+    ((Widget *) bar)->y = LINES - 1;
     
     run_dlg (dlg);
     if (dlg->ret_value == B_ENTER)
@@ -687,8 +702,8 @@ tree_box (char *current_dir)
 #ifdef USE_VFS
 
 #if defined(USE_NETCODE)
-#define VFSY 16
-#define VFS_WIDGETBASE 9
+#define VFSY 17
+#define VFS_WIDGETBASE 10
 #else
 #define VFSY 8
 #define VFS_WIDGETBASE 0
@@ -704,40 +719,43 @@ static char *ret_directory_timeout;
 static char *ret_ftp_proxy;
 static int ret_use_netrc;
 static int ret_ftpfs_use_passive_connections;
+static int ret_ftpfs_use_passive_connections_over_proxy;
 #endif
 
 static QuickWidget confvfs_widgets [] = {
 { quick_button,   30,  VFSX,    VFSY - 3, VFSY, N_("&Cancel"),
-      0, B_CANCEL, 0, 0, "button-cancel" },
+      0, B_CANCEL, 0, 0, NULL },
 { quick_button,   12, VFSX,    VFSY - 3, VFSY, N_("&OK"),
-      0, B_ENTER, 0, 0, "button-ok" },
+      0, B_ENTER, 0, 0, NULL },
 #if defined(USE_NETCODE)
+{ quick_checkbox,  4, VFSX, 12, VFSY, N_("Use passive mode over pro&xy"), 0, 0,
+      &ret_ftpfs_use_passive_connections_over_proxy, 0, NULL },
 { quick_checkbox,  4, VFSX, 11, VFSY, N_("Use &passive mode"), 0, 0,
-      &ret_ftpfs_use_passive_connections, 0, "check-use-passive-mode" },
+      &ret_ftpfs_use_passive_connections, 0, NULL },
 { quick_checkbox,  4, VFSX, 10, VFSY, N_("&Use ~/.netrc"), 0, 0,
-      &ret_use_netrc, 0, "check-use-netrc" },
+      &ret_use_netrc, 0, NULL },
 { quick_input,     4, VFSX, 9, VFSY, "", 48, 0, 0, &ret_ftp_proxy,
       "input-ftp-proxy" },
 { quick_checkbox,  4, VFSX, 8, VFSY, N_("&Always use ftp proxy"), 0, 0,
-      &ftpfs_always_use_proxy, 0, "check-ftp-proxy" },
+      &ftpfs_always_use_proxy, 0, NULL },
 { quick_label,    49, VFSX, 7, VFSY, N_("sec"),
-      0, 0, 0, 0, "label-sec" },
+      0, 0, 0, 0, NULL },
 { quick_input,    38, VFSX, 7, VFSY, "", 10, 0, 0, &ret_directory_timeout,
       "input-timeout" },
 { quick_label,     4, VFSX, 7, VFSY, N_("ftpfs directory cache timeout:"),
-      0, 0, 0, 0, "label-cache"},
+      0, 0, 0, 0, NULL },
 { quick_input,     4, VFSX, 6, VFSY, "", 48, 0, 0, &ret_passwd,
       "input-passwd" },
 { quick_label,     4, VFSX, 5, VFSY, N_("ftp anonymous password:"),
-      0, 0, 0, 0, "label-pass"},
+      0, 0, 0, 0, NULL },
 #endif
 { quick_label,    49, VFSX, 3, VFSY, "sec",
-      0, 0, 0, 0, "label-sec2" },
+      0, 0, 0, 0, NULL },
 { quick_input,    38, VFSX, 3, VFSY, "", 10, 0, 0, &ret_timeout, 
       "input-timo-vfs" },
 { quick_label,    4,  VFSX, 3, VFSY, N_("Timeout for freeing VFSs:"), 
-      0, 0, 0, 0, "label-vfs" },
-{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
+      0, 0, 0, 0, NULL },
+NULL_QuickWidget
 };
 
 static QuickDialog confvfs_dlg =
@@ -753,10 +771,11 @@ configure_vfs (void)
 
     ret_use_netrc = use_netrc;
     ret_ftpfs_use_passive_connections = ftpfs_use_passive_connections;
+    ret_ftpfs_use_passive_connections_over_proxy = ftpfs_use_passive_connections_over_proxy;
     g_snprintf(buffer3, sizeof (buffer3), "%i", ftpfs_directory_timeout);
-    confvfs_widgets[7].text = buffer3;
-    confvfs_widgets[9].text = ftpfs_anonymous_passwd;
-    confvfs_widgets[4].text = ftpfs_proxy_host;
+    confvfs_widgets[8].text = buffer3;
+    confvfs_widgets[10].text = ftpfs_anonymous_passwd;
+    confvfs_widgets[5].text = ftpfs_proxy_host;
 #endif
     g_snprintf (buffer2, sizeof (buffer2), "%i", vfs_timeout);
     confvfs_widgets [3 + VFS_WIDGETBASE].text = buffer2;
@@ -769,12 +788,12 @@ configure_vfs (void)
 #if defined(USE_NETCODE)
 	g_free (ftpfs_anonymous_passwd);
 	ftpfs_anonymous_passwd = ret_passwd;
-	if (ftpfs_proxy_host)
-	    g_free (ftpfs_proxy_host);
+	g_free (ftpfs_proxy_host);
 	ftpfs_proxy_host = ret_ftp_proxy;
 	ftpfs_directory_timeout = atoi(ret_directory_timeout);
 	use_netrc = ret_use_netrc;
 	ftpfs_use_passive_connections = ret_ftpfs_use_passive_connections;
+	ftpfs_use_passive_connections_over_proxy = ret_ftpfs_use_passive_connections_over_proxy;
 	g_free (ret_directory_timeout);
 #endif
     }
@@ -788,8 +807,8 @@ cd_dialog (void)
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
 	{ quick_input,  6, 57, 2, 0, "", 50, 0, 0, 0, "input" },
-	{ quick_label,  3, 57, 2, 0, "",  0, 0, 0, 0, "label" },
-	{ 0 }
+	{ quick_label,  3, 57, 2, 0, "",  0, 0, 0, 0, NULL },
+	NULL_QuickWidget
     };
     char *my_str;
     int len;
@@ -824,40 +843,46 @@ cd_dialog (void)
 }
 
 void
-symlink_dialog (char *existing, char *new, char **ret_existing, char **ret_new)
+symlink_dialog (const char *existing, const char *new, char **ret_existing,
+		char **ret_new)
 {
     QuickDialog Quick_input;
-    QuickWidget quick_widgets [] = {
-    { quick_input,  4, 80, 5, 8, "", 58, 0, 0, 0, "input-1" },
-    { quick_label,  4, 80, 4, 8, "", 0, 0, 0, 0, "label-1" },
-    { quick_input,  4, 80, 3, 8, "", 58, 0, 0, 0, "input-2" },
-    { quick_label,  4, 80, 2, 8, "", 0, 0, 0, 0, "label-2" },
-    { 0 } };
-    
-    Quick_input.xlen  = 64;
-    Quick_input.ylen  = 8;
-    Quick_input.title = _("Symbolic link");
-    Quick_input.help  = "[File Menu]";
-    Quick_input.i18n  = 0;
-    quick_widgets [0].text = new;
-    quick_widgets [1].text = _("Symbolic link filename:");
-    quick_widgets [2].text = existing;
-    quick_widgets [3].text = _("Existing filename (filename symlink will point to):");
+    QuickWidget quick_widgets[] = {
+	{quick_button, 50, 80, 6, 8, N_("&Cancel"), 0, B_CANCEL, 0, 0,
+	 NULL},
+	{quick_button, 16, 80, 6, 8, N_("&OK"), 0, B_ENTER, 0, 0, NULL},
+	{quick_input, 4, 80, 5, 8, "", 58, 0, 0, 0, "input-1"},
+	{quick_label, 4, 80, 4, 8, N_("Symbolic link filename:"), 0, 0, 0,
+	 0, NULL},
+	{quick_input, 4, 80, 3, 8, "", 58, 0, 0, 0, "input-2"},
+	{quick_label, 4, 80, 2, 8,
+	 N_("Existing filename (filename symlink will point to):"), 0, 0,
+	 0, 0, NULL},
+	NULL_QuickWidget
+    };
+
+    Quick_input.xlen = 64;
+    Quick_input.ylen = 12;
+    Quick_input.title = N_("Symbolic link");
+    Quick_input.help = "[File Menu]";
+    Quick_input.i18n = 0;
+    quick_widgets[2].text = new;
+    quick_widgets[4].text = existing;
     Quick_input.xpos = -1;
-    quick_widgets [0].str_result = ret_new;
-    quick_widgets [2].str_result = ret_existing;
-    
+    quick_widgets[2].str_result = ret_new;
+    quick_widgets[4].str_result = ret_existing;
+
     Quick_input.widgets = quick_widgets;
-    if (quick_dialog (&Quick_input) == B_CANCEL){
-        *ret_new = NULL;
-        *ret_existing = NULL;
+    if (quick_dialog (&Quick_input) == B_CANCEL) {
+	*ret_new = NULL;
+	*ret_existing = NULL;
     }
 }
 
 #ifdef WITH_BACKGROUND
-#define B_STOP   B_USER+1
-#define B_RESUME B_USER+2
-#define B_KILL   B_USER+3
+#define B_STOP   (B_USER+1)
+#define B_RESUME (B_USER+2)
+#define B_KILL   (B_USER+3)
 
 static int JOBS_X = 60;
 #define JOBS_Y 15
@@ -867,7 +892,7 @@ static Dlg_head *jobs_dlg;
 static void
 jobs_fill_listbox (void)
 {
-    static char *state_str [2];
+    static const char *state_str [2];
     TaskList *tl = task_list;
 
     if (!state_str [0]){
@@ -878,7 +903,7 @@ jobs_fill_listbox (void)
     while (tl){
 	char *s;
 
-	s = g_strconcat (state_str [tl->state], " ", tl->info, NULL);
+	s = g_strconcat (state_str [tl->state], " ", tl->info, (char *) NULL);
 	listbox_add_item (bg_list, LISTBOX_APPEND_AT_END, 0, s, (void *) tl);
 	g_free (s);
 	tl = tl->next;
@@ -886,7 +911,7 @@ jobs_fill_listbox (void)
 }
 	
 static int
-task_cb (int action, void *ignored)
+task_cb (int action)
 {
     TaskList *tl;
     int sig = 0;
@@ -925,18 +950,17 @@ task_cb (int action, void *ignored)
 
 static struct 
 {
-	char* name;
+	const char* name;
 	int xpos;
 	int value;
-	int (*callback)(int, void *);
-	char* tkname;
+	int (*callback)(int);
 } 
 job_buttons [] =
 {
-	{N_("&Stop"),   3,  B_STOP,   task_cb, "button-stop"},
-	{N_("&Resume"), 12, B_RESUME, task_cb, "button-cont"},
-	{N_("&Kill"),   23, B_KILL,   task_cb, "button-kill"},
-	{N_("&OK"),     35, B_CANCEL, NULL,    "button-ok"},
+	{N_("&Stop"),   3,  B_STOP,   task_cb},
+	{N_("&Resume"), 12, B_RESUME, task_cb},
+	{N_("&Kill"),   23, B_KILL,   task_cb},
+	{N_("&OK"),     35, B_CANCEL, NULL   }
 };
 
 void
@@ -973,9 +997,9 @@ jobs_cmd (void)
 
     jobs_dlg = create_dlg (0, 0, JOBS_Y, JOBS_X, dialog_colors, NULL,
 			   "[Background jobs]", _("Background Jobs"),
-			   DLG_CENTER);
+			   DLG_CENTER | DLG_REVERSE);
 
-    bg_list = listbox_new (2, 3, JOBS_X-7, JOBS_Y-9, listbox_nothing, 0, "listbox");
+    bg_list = listbox_new (2, 3, JOBS_X-7, JOBS_Y-9, 0);
     add_widget (jobs_dlg, bg_list);
 
 	i = n_buttons;
@@ -984,8 +1008,7 @@ jobs_cmd (void)
 		add_widget (jobs_dlg, button_new (JOBS_Y-4, 
 			job_buttons [i].xpos, job_buttons [i].value,
 			NORMAL_BUTTON, job_buttons [i].name, 
-			job_buttons [i].callback, 0,
-			job_buttons [i].tkname));
+			job_buttons [i].callback));
 	}
 	
     /* Insert all of task information in the list */
@@ -1004,8 +1027,8 @@ vfs_smb_get_authinfo (const char *host, const char *share, const char *domain,
     static int dialog_x = 44;
     enum { b0 = 3, dialog_y = 12};
     struct smb_authinfo *return_value;
-    static char* labs[] = {N_("Domain:"), N_("Username:"), N_("Password:")};
-    static char* buts[] = {N_("&OK"), N_("&Cancel")};
+    static const char* labs[] = {N_("Domain:"), N_("Username:"), N_("Password:")};
+    static const char* buts[] = {N_("&OK"), N_("&Cancel")};
     static int ilen = 30, istart = 14;
     static int b2 = 30;
     char *title;
@@ -1058,28 +1081,28 @@ vfs_smb_get_authinfo (const char *host, const char *share, const char *domain,
     title = g_strdup_printf (_("Password for \\\\%s\\%s"), host, share);
 
     auth_dlg = create_dlg (0, 0, dialog_y, dialog_x, dialog_colors, NULL,
-			   "[Smb Authinfo]", title, DLG_CENTER);
+			   "[Smb Authinfo]", title, DLG_CENTER | DLG_REVERSE);
 
     g_free (title);
 
-    in_user  = input_new (5, istart, INPUT_COLOR, ilen, user, "auth_name");
+    in_user  = input_new (5, istart, INPUT_COLOR, ilen, user, "auth_name", INPUT_COMPLETE_DEFAULT);
     add_widget (auth_dlg, in_user);
 
-    in_domain = input_new (3, istart, INPUT_COLOR, ilen, domain, "auth_domain");
+    in_domain = input_new (3, istart, INPUT_COLOR, ilen, domain, "auth_domain", INPUT_COMPLETE_DEFAULT);
     add_widget (auth_dlg, in_domain);
     add_widget (auth_dlg, button_new (9, b2, B_CANCEL, NORMAL_BUTTON,
-                 buts[1], 0 ,0, "cancel"));
+                 buts[1], 0));
     add_widget (auth_dlg, button_new (9, b0, B_ENTER, DEFPUSH_BUTTON,
-                 buts[0], 0, 0, "ok"));
+                 buts[0], 0));
 
-    in_password  = input_new (7, istart, INPUT_COLOR, ilen, "", "auth_password");
+    in_password  = input_new (7, istart, INPUT_COLOR, ilen, "", "auth_password", INPUT_COMPLETE_DEFAULT);
     in_password->completion_flags = 0;
     in_password->is_password = 1;
     add_widget (auth_dlg, in_password);
 
-    add_widget (auth_dlg, label_new (7, 3, labs[2], "label-passwd"));
-    add_widget (auth_dlg, label_new (5, 3, labs[1], "label-user"));
-    add_widget (auth_dlg, label_new (3, 3, labs[0], "label-domain"));
+    add_widget (auth_dlg, label_new (7, 3, labs[2]));
+    add_widget (auth_dlg, label_new (5, 3, labs[1]));
+    add_widget (auth_dlg, label_new (3, 3, labs[0]));
 
     run_dlg (auth_dlg);
 
