@@ -1,5 +1,6 @@
 /* Directory routines
-   Copyright (C) 1994 Miguel de Icaza.
+   Copyright (C) 1994, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+   2006, 2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,10 +17,12 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
-#include <string.h>
+
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <errno.h>
+#include <string.h>
+
 #include <sys/stat.h>
 
 #include "global.h"
@@ -43,7 +46,10 @@ static int reverse = 1;
 /* Are the files sorted case sensitively? */
 static int case_sensitive = OS_SORT_CASE_SENSITIVE_DEFAULT;
 
-#define MY_ISDIR(x) ( (S_ISDIR (x->st.st_mode) || x->f.link_to_dir) ? 1 : 0)
+/* Are the exec_bit files top in list*/
+static int exec_first = 1;
+
+#define MY_ISDIR(x) ( (is_exe (x->st.st_mode) && !(S_ISDIR (x->st.st_mode) || x->f.link_to_dir) && (exec_first == 1)) ? 1 : ( (S_ISDIR (x->st.st_mode) || x->f.link_to_dir) ? 2 : 0) )
 
 sort_orders_t sort_orders [SORT_TYPES_TOTAL] = {
     { N_("&Unsorted"),    unsorted },
@@ -51,17 +57,9 @@ sort_orders_t sort_orders [SORT_TYPES_TOTAL] = {
     { N_("&Extension"),   sort_ext },
     { N_("&Modify time"), sort_time },
     { N_("&Access time"), sort_atime },
-    { N_("&Change time"), sort_ctime },
+    { N_("C&Hange time"), sort_ctime },
     { N_("&Size"),        sort_size },
     { N_("&Inode"),       sort_inode },
-
-    /* New sort orders */
-    { N_("&Type"),        sort_type },
-    { N_("&Links"),       sort_links },
-    { N_("N&GID"),        sort_ngid },
-    { N_("N&UID"),        sort_nuid },
-    { N_("&Owner"),       sort_owner },
-    { N_("&Group"),       sort_group }
 };
 
 #ifdef HAVE_STRCOLL
@@ -109,6 +107,8 @@ static int string_sortcomp (const char *str1, const char *str2)
 int
 unsorted (const file_entry *a, const file_entry *b)
 {
+    (void) a;
+    (void) b;
     return 0;
 }
 
@@ -143,36 +143,20 @@ sort_ext (const file_entry *a, const file_entry *b)
 	return bd-ad;
 }
 
-int
-sort_owner (const file_entry *a, const file_entry *b)
-{
-    int ad = MY_ISDIR (a);
-    int bd = MY_ISDIR (b);
-
-    if (ad == bd || mix_all_files)
-	return string_sortcomp (get_owner (a->st.st_uid), get_owner (a->st.st_uid)) * reverse;
-    return bd-ad;
-}
-
-int
-sort_group (const file_entry *a, const file_entry *b)
-{
-    int ad = MY_ISDIR (a);
-    int bd = MY_ISDIR (b);
-
-    if (ad == bd || mix_all_files)
-	return string_sortcomp (get_group (a->st.st_gid), get_group (a->st.st_gid)) * reverse;
-    return bd-ad;
-}
-
 int
 sort_time (const file_entry *a, const file_entry *b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
 
-    if (ad == bd || mix_all_files)
-	return (a->st.st_mtime - b->st.st_mtime) * reverse;
+    if (ad == bd || mix_all_files) {
+	int result = a->st.st_mtime < b->st.st_mtime ? -1 :
+		     a->st.st_mtime > b->st.st_mtime;
+	if (result != 0)
+	    return result * reverse;
+	else
+	    return sort_name (a, b);
+    }
     else
 	return bd-ad;
 }
@@ -183,8 +167,14 @@ sort_ctime (const file_entry *a, const file_entry *b)
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
 
-    if (ad == bd || mix_all_files)
-	return (a->st.st_ctime - b->st.st_ctime) * reverse;
+    if (ad == bd || mix_all_files) {
+	int result = a->st.st_ctime < b->st.st_ctime ? -1 :
+		     a->st.st_ctime > b->st.st_ctime;
+	if (result != 0)
+	    return result * reverse;
+	else
+	    return sort_name (a, b);
+    }
     else
 	return bd-ad;
 }
@@ -195,8 +185,14 @@ sort_atime (const file_entry *a, const file_entry *b)
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
 
-    if (ad == bd || mix_all_files)
-	return (a->st.st_atime - b->st.st_atime) * reverse;
+    if (ad == bd || mix_all_files) {
+	int result = a->st.st_atime < b->st.st_atime ? -1 :
+		     a->st.st_atime > b->st.st_atime;
+	if (result != 0)
+	    return result * reverse;
+	else
+	    return sort_name (a, b);
+    }
     else
 	return bd-ad;
 }
@@ -218,110 +214,36 @@ sort_size (const file_entry *a, const file_entry *b)
 {
     int ad = MY_ISDIR (a);
     int bd = MY_ISDIR (b);
+    int result = 0;
 
     if (ad != bd && !mix_all_files)
 	return bd - ad;
 
-    return (2 * (b->st.st_size > a->st.st_size) - 1) * reverse;
-}
-
-int
-sort_links (const file_entry *a, const file_entry *b)
-{
-    int ad = MY_ISDIR (a);
-    int bd = MY_ISDIR (b);
-
-    if (ad == bd || mix_all_files)
-	return (b->st.st_nlink - a->st.st_nlink) * reverse;
+    result = a->st.st_size < b->st.st_size ? -1 :
+	     a->st.st_size > b->st.st_size;
+    if (result != 0)
+	return result * reverse;
     else
-	return bd-ad;
-}
-
-int
-sort_ngid (const file_entry *a, const file_entry *b)
-{
-    int ad = MY_ISDIR (a);
-    int bd = MY_ISDIR (b);
-
-    if (ad == bd || mix_all_files)
-	return (b->st.st_gid - a->st.st_gid) * reverse;
-    else
-	return bd-ad;
-}
-
-int
-sort_nuid (const file_entry *a, const file_entry *b)
-{
-    int ad = MY_ISDIR (a);
-    int bd = MY_ISDIR (b);
-
-    if (ad == bd || mix_all_files)
-	return (b->st.st_uid - a->st.st_uid) * reverse;
-    else
-	return bd-ad;
-}
-
-inline static int
-file_type_to_num (const file_entry *fe)
-{
-    const struct stat *s = &fe->st;
-
-    if (S_ISDIR (s->st_mode))
-	return 0;
-    if (S_ISLNK (s->st_mode)){
-	if (fe->f.link_to_dir)
-	    return 1;
-	if (fe->f.stale_link)
-	    return 2;
-	else
-	    return 3;
-    }
-    if (S_ISSOCK (s->st_mode))
-	return 4;
-    if (S_ISCHR (s->st_mode))
-	return 5;
-    if (S_ISBLK (s->st_mode))
-	return 6;
-    if (S_ISFIFO (s->st_mode))
-	return 7;
-    if (S_ISNAM (s->st_mode)) /* Special named files will be shown as block devices */
-	return 6;
-    if (is_exe (s->st_mode))
-	return 8;
-    return 9;
-}
-
-int
-sort_type (const file_entry *a, const file_entry *b)
-{
-    int aa  = file_type_to_num (a);
-    int bb  = file_type_to_num (b);
-
-    return bb-aa;
+	return sort_name (a, b);
 }
 
 
 void
-do_sort (dir_list *list, sortfn *sort, int top, int reverse_f, int case_sensitive_f)
+do_sort (dir_list *list, sortfn *sort, int top, int reverse_f, int case_sensitive_f, int exec_first_f)
 {
-    int i;
     int dot_dot_found = 0;
-    file_entry tmp_fe;
 
-    for (i = 0; i < top + 1; i++) {             /* put ".." first in list */
-	if (!strcmp (list->list [i].fname, "..")) {
-	    dot_dot_found = 1;
-            if (i > 0) {                        /* swap [i] and [0] */
-                memcpy (&tmp_fe, &(list->list [0]), sizeof (file_entry));
-                memcpy (&(list->list [0]), &(list->list [i]), sizeof (file_entry));
-                memcpy (&(list->list [i]), &tmp_fe, sizeof (file_entry));
-            }
-            break;
-        }
-    }
+    if (top == 0)
+	return;
+
+    /* If there is an ".." entry the caller must take care to
+       ensure that it occupies the first list element. */
+    if (!strcmp (list->list [0].fname, ".."))
+	dot_dot_found = 1;
 
     reverse = reverse_f ? -1 : 1;
     case_sensitive = case_sensitive_f;
+    exec_first = exec_first_f;
     qsort (&(list->list) [dot_dot_found],
 	   top + 1 - dot_dot_found, sizeof (file_entry), sort);
 }
@@ -464,7 +386,7 @@ handle_path (dir_list *list, const char *path,
 
 int
 do_load_dir (const char *path, dir_list *list, sortfn *sort, int reverse,
-	     int case_sensitive, const char *filter)
+	     int case_sensitive, int exec_ff, const char *filter)
 {
     DIR *dirp;
     struct dirent *dp;
@@ -472,12 +394,20 @@ do_load_dir (const char *path, dir_list *list, sortfn *sort, int reverse,
     int next_free = 0;
     struct stat st;
 
+    /* ".." (if any) must be the first entry in the list */
+    if (set_zero_dir (list) == 0)
+	return next_free;
+    next_free++;
+
     dirp = mc_opendir (path);
     if (!dirp) {
 	message (1, MSG_ERROR, _("Cannot read directory contents"));
-	return set_zero_dir (list);
+	return next_free;
     }
     tree_store_start_check (path);
+    /* Do not add a ".." entry to the root directory */
+    if (!strcmp (path, "/"))
+	next_free--;
     while ((dp = mc_readdir (dirp))) {
 	status =
 	    handle_dirent (list, filter, dp, &st, next_free, &link_to_dir,
@@ -502,12 +432,7 @@ do_load_dir (const char *path, dir_list *list, sortfn *sort, int reverse,
     }
 
     if (next_free) {
-	/* Add ".." except the root directory */
-	if (strcmp (path, "/") != 0)
-	    add_dotdot_to_list (list, next_free++);
-	do_sort (list, sort, next_free - 1, reverse, case_sensitive);
-    } else {
-	next_free = set_zero_dir (list);
+	do_sort (list, sort, next_free - 1, reverse, case_sensitive, exec_ff);
     }
 
     mc_closedir (dirp);
@@ -563,7 +488,7 @@ alloc_dir_copy (int size)
 /* If filter is null, then it is a match */
 int
 do_reload_dir (const char *path, dir_list *list, sortfn *sort, int count,
-	       int rev, int case_sensitive, const char *filter)
+	       int rev, int case_sensitive, int exec_ff, const char *filter)
 {
     DIR *dirp;
     struct dirent *dp;
@@ -598,6 +523,17 @@ do_reload_dir (const char *path, dir_list *list, sortfn *sort, int count,
 	}
     }
 
+    /* Add ".." except to the root directory. The ".." entry
+       (if any) must be the first in the list. */
+    if (strcmp (path, "/") != 0) {
+	if (set_zero_dir (list) == 0) {
+	    clean_dir (list, count);
+	    clean_dir (&dir_copy, count);
+	    return next_free;
+	}
+	next_free++;
+    }
+
     while ((dp = mc_readdir (dirp))) {
 	status =
 	    handle_dirent (list, filter, dp, &st, next_free, &link_to_dir,
@@ -649,12 +585,8 @@ do_reload_dir (const char *path, dir_list *list, sortfn *sort, int count,
     tree_store_end_check ();
     g_hash_table_destroy (marked_files);
     if (next_free) {
-	/* Add ".." except the root directory */
-	if (strcmp (path, "/") != 0)
-	    add_dotdot_to_list (list, next_free++);
-	do_sort (list, sort, next_free - 1, rev, case_sensitive);
-    } else
-	next_free = set_zero_dir (list);
+	do_sort (list, sort, next_free - 1, rev, case_sensitive, exec_ff);
+    }
     clean_dir (&dir_copy, count);
     return next_free;
 }
