@@ -1,5 +1,5 @@
 /* Execution routines for GNU Midnight Commander
-   Copyright (C) 1994-2003 The Free Software Foundation
+   Copyright (C) 2003, 2004, 2005, 2007 Free Software Foundation, Inc.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,9 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
-#include <signal.h>	/* kill() */
+
+#include <signal.h>
+
 #include "global.h"
 #include "tty.h"
 #include "win.h"
@@ -107,6 +109,7 @@ do_execute (const char *shell, const char *command, int flags)
 
     if (!use_subshell && command && !(flags & EXECUTE_INTERNAL)) {
 	printf ("%s%s\n", prompt, command);
+	fflush (stdout);
     }
 #ifdef HAVE_SUBSHELL_SUPPORT
     if (use_subshell && !(flags & EXECUTE_INTERNAL)) {
