@@ -1,6 +1,6 @@
 /* Hypertext file browser.
-   Copyright (C) 1994, 1995 Miguel de Icaza.
-   Copyright (C) 1994, 1995 Janne Kukonlehto
+   Copyright (C) 1994, 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+   2005, 2006, 2007 Free Software Foundation, Inc.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,12 +14,17 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+*/
 
+/*
    Implements the hypertext file viewer.
    The hypertext file is a file that may have one or more nodes.  Each
    node ends with a ^D character and starts with a bracket, then the
-   name of the node and then a closing bracket.
+   name of the node and then a closing bracket. Right after the closing
+   bracket a newline is placed. This newline is not to be displayed by
+   the help viewer and must be skipped - its sole purpose is to faciliate
+   the work of the people managing the help file template (xnc.hlp) .
 
    Links in the hypertext file are specified like this: the text that
    will be highlighted should have a leading ^A, then it comes the
@@ -37,96 +42,71 @@
 */
 
 #include <config.h>
-#include "tty.h"
+
+#include <errno.h>
 #include <stdio.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <errno.h>
+
 #include "global.h"
+#include "tty.h"
 #include "color.h"
-#include "dialog.h"
 #include "win.h"
 #include "mouse.h"
-#include "key.h"	/* For mi_getch() */
+#include "key.h"		/* For mi_getch() */
 #include "help.h"
-#include "layout.h"		/* keybar_visible */
-#include "x.h"
-#include "dlg.h"		/* For Dlg_head */
+#include "dialog.h"		/* For Dlg_head */
 #include "widget.h"		/* For Widget */
+#include "wtools.h"		/* For common_dialog_repaint() */
 
 #define MAXLINKNAME 80
 #define HISTORY_SIZE 20
-#define HELP_WINDOW_WIDTH 62
+#define HELP_WINDOW_WIDTH (HELP_TEXT_WIDTH + 4)
 
-/* "$Id: help.c,v 1.14 2001/08/14 00:55:38 proskin Exp $" */
+#define STRING_LINK_START	"\01"
+#define STRING_LINK_POINTER	"\02"
+#define STRING_LINK_END		"\03"
+#define STRING_NODE_END		"\04"
 
 static char *data;		/* Pointer to the loaded data file */
-static int help_lines = 18;	/* Lines in help viewer */
+static int help_lines;		/* Lines in help viewer */
 static int  history_ptr;	/* For the history queue */
-static char *main_node;		/* The main node */
-static char *last_shown = 0;	/* Last byte shown in a screen */
+static const char *main_node;	/* The main node */
+static const char *last_shown = NULL;	/* Last byte shown in a screen */
 static int end_of_node = 0;	/* Flag: the last character of the node shown? */
-static char *currentpoint, *startpoint;
-static char *selected_item;
+static const char *currentpoint;
+static const char *selected_item;
 
 /* The widget variables */
 static Dlg_head *whelp;
 
 static struct {
-    char *page;			/* Pointer to the selected page */
-    char *link;			/* Pointer to the selected link */
+    const char *page;		/* Pointer to the selected page */
+    const char *link;		/* Pointer to the selected link */
 } history [HISTORY_SIZE];
 
 /* Link areas for the mouse */
 typedef struct Link_Area {
     int x1, y1, x2, y2;
-    char *link_name;
+    const char *link_name;
     struct Link_Area *next;
 } Link_Area;
 
 static Link_Area *link_area = NULL;
 static int inside_link_area = 0;
 
-static int help_callback (struct Dlg_head *h, int id, int msg);
-
-#ifdef HAS_ACS_AS_PCCHARS
-static struct {
-    int acscode;
-    int pccode;
-} acs2pc_table [] = {
-    { 'q',  0xC4 },
-    { 'x',  0xB3 },
-    { 'l',  0xDA },
-    { 'k',  0xBF },
-    { 'm',  0xC0 },
-    { 'j',  0xD9 },
-    { 'a',  0xB0 },
-    { 'u',  0xB4 },
-    { 't',  0xC3 },
-    { 'w',  0xC2 },
-    { 'v',  0xC1 },
-    { 'n',  0xC5 },
-    { 0, 0 } };
-
-static int acs2pc (int acscode)
-{
-    int i;
-
-    for (i = 0; acs2pc_table[i].acscode != 0; i++)
-	if (acscode == acs2pc_table[i].acscode) {
-	    return acs2pc_table[i].pccode;
-	}
-    return 0;
-}
-#endif
+static cb_ret_t help_callback (struct Dlg_head *h, dlg_msg_t, int parm);
 
 /* returns the position where text was found in the start buffer */
 /* or 0 if not found */
-static char *
-search_string (char *start, char *text)
+static const char *
+search_string (const char *start, const char *text)
 {
-    char *d = text;
-    char *e = start;
+    const char *result = NULL;
+    char *local_text = g_strdup (text);
+    char *d = local_text;
+    const char *e = start;
 
     /* fmt sometimes replaces a space with a newline in the help file */
     /* Replace the newlines in the link name with spaces to correct the situation */
@@ -136,23 +116,27 @@ search_string (char *start, char *text)
 	d++;
     }
     /* Do search */
-    for (d = text; *e; e++){
+    for (d = local_text; *e; e++){
 	if (*d == *e)
 	    d++;
 	else
-	    d = text;
-	if (!*d)
-	    return e+1;
+	    d = local_text;
+	if (!*d) {
+	    result = e + 1;
+	    goto cleanup;
+	}
     }
-    return 0;
+cleanup:
+    g_free (local_text);
+    return result;
 }
 
 /* Searches text in the buffer pointed by start.  Search ends */
 /* if the CHAR_NODE_END is found in the text.  Returns 0 on failure */
-static char *search_string_node (char *start, char *text)
+static const char *search_string_node (const char *start, const char *text)
 {
-    char *d = text;
-    char *e = start;
+    const char *d = text;
+    const char *e = start;
 
     if (!start)
 	return 0;
@@ -170,9 +154,9 @@ static char *search_string_node (char *start, char *text)
 
 /* Searches the_char in the buffer pointer by start and searches */
 /* it can search forward (direction = 1) or backward (direction = -1) */
-static char *search_char_node (char *start, char the_char, int direction)
+static const char *search_char_node (const char *start, char the_char, int direction)
 {
-    char *e;
+    const char *e;
 
     e = start;
     
@@ -184,9 +168,9 @@ static char *search_char_node (char *start, char the_char, int direction)
 }
 
 /* Returns the new current pointer when moved lines lines */
-static char *move_forward2 (char *c, int lines)
+static const char *move_forward2 (const char *c, int lines)
 {
-    char *p;
+    const char *p;
     int  line;
 
     currentpoint = c;
@@ -199,9 +183,9 @@ static char *move_forward2 (char *c, int lines)
     return currentpoint = c;
 }
 
-static char *move_backward2 (char *c, int lines)
+static const char *move_backward2 (const char *c, int lines)
 {
-    char *p;
+    const char *p;
     int line;
 
     currentpoint = c;
@@ -211,7 +195,7 @@ static char *move_backward2 (char *c, int lines)
 	    /* We reached the beginning of the node */
 	    /* Skip the node headers */
 	    while (*p != ']') p++;
-	    return currentpoint = p + 2;
+	    return currentpoint = p + 2; /* Skip the newline following the start of the node */
 	}
 	if (*(p - 1) == '\n')
 	    line++;
@@ -233,17 +217,17 @@ static void move_backward (int i)
     currentpoint = move_backward2 (currentpoint, ++i);
 }
 
-static void move_to_top (int dummy)
+static void move_to_top (void)
 {
     while (currentpoint > data && *currentpoint != CHAR_NODE_END)
 	currentpoint--;
     while (*currentpoint != ']')
 	currentpoint++;
-    currentpoint = currentpoint + 1;
+    currentpoint = currentpoint + 2; /* Skip the newline following the start of the node */
     selected_item = NULL;
 }
 
-static void move_to_bottom (int dummy)
+static void move_to_bottom (void)
 {
     while (*currentpoint && *currentpoint != CHAR_NODE_END)
 	currentpoint++;
@@ -251,10 +235,10 @@ static void move_to_bottom (int dummy)
     move_backward (help_lines - 1);
 }
 
-char *help_follow_link (char *start, char *selected_item)
+static const char *help_follow_link (const char *start, const char *selected_item)
 {
     char link_name [MAXLINKNAME];
-    char *p;
+    const char *p;
     int  i = 0;
 
     if (!selected_item)
@@ -269,15 +253,19 @@ char *help_follow_link (char *start, char *selected_item)
 	link_name [i-1] = ']';
 	link_name [i] = 0;
 	p = search_string (data, link_name);
-	if (p)
+	if (p) {
+	    p += 1; /* Skip the newline following the start of the node */
 	    return p;
+	}
     }
-    return _(" Help file format error\n\x4");	/*  */
+
+    /* Create a replacement page with the error message */
+    return _(" Help file format error\n");
 }
 
-static char *select_next_link (char *start, char *current_link)
+static const char *select_next_link (const char *current_link)
 {
-    char *p;
+    const char *p;
 
     if (!current_link)
 	return 0;
@@ -291,27 +279,20 @@ static char *select_next_link (char *start, char *current_link)
     return p - 1;
 }
 
-static char *select_prev_link (char *start, char *current_link)
+static const char *select_prev_link (const char *current_link)
 {
-    char *p;
-
     if (!current_link)
 	return 0;
-    
-    p = current_link - 1;
-    if (p <= start)
-	return 0;
-    
-    p = search_char_node (p, CHAR_LINK_START, -1);
-    return p;
+
+    return search_char_node (current_link - 1, CHAR_LINK_START, -1);
 }
 
-static void start_link_area (int x, int y, char *link_name)
+static void start_link_area (int x, int y, const char *link_name)
 {
     Link_Area *new;
 
     if (inside_link_area)
-	message (0, _(" Warning "), _(" Internal bug: Double start of link area "));
+	message (0, _("Warning"), _(" Internal bug: Double start of link area "));
 
     /* Allocate memory for a new link area */
     new = g_new (Link_Area, 1);
@@ -351,15 +332,16 @@ static void clear_link_areas (void)
     inside_link_area = 0;
 }
 
-static void show (Dlg_head *h, char *paint_start)
+static void help_show (Dlg_head *h, const char *paint_start)
 {
-    char *p;
+    const char *p;
     int  col, line, c;
     int  painting = 1;
     int acs;			/* Flag: Alternate character set active? */
     int repeat_paint;
     int active_col, active_line;/* Active link position */
 
+    attrset (HELP_NORMAL_COLOR);
     do {
 	
 	line = col = acs = active_col = active_line = repeat_paint = 0;
@@ -368,8 +350,8 @@ static void show (Dlg_head *h, char *paint_start)
 	if (selected_item < paint_start)
 	    selected_item = NULL;
 	
-	for (p = paint_start; *p != CHAR_NODE_END && line < help_lines; p++){
-	    c = *p;
+	for (p = paint_start; *p && *p != CHAR_NODE_END && line < help_lines; p++) {
+	    c = (unsigned char)*p;
 	    switch (c){
 	    case CHAR_LINK_START:
 		if (selected_item == NULL)
@@ -404,13 +386,13 @@ static void show (Dlg_head *h, char *paint_start)
 		addstr (VERSION);
 		col += strlen (VERSION);
 		break;
-	    case CHAR_BOLD_ON:
+	    case CHAR_FONT_BOLD:
 		attrset (HELP_BOLD_COLOR);
 		break;
-	    case CHAR_ITALIC_ON:
+	    case CHAR_FONT_ITALIC:
 		attrset (HELP_ITALIC_COLOR);
 		break;
-	    case CHAR_BOLD_OFF:
+	    case CHAR_FONT_NORMAL:
 		attrset (HELP_NORMAL_COLOR);
 		break;
 	    case '\n':
@@ -420,16 +402,6 @@ static void show (Dlg_head *h, char *paint_start)
 	    case '\t':
 		col = (col/8 + 1) * 8;
 		break;
-	    case CHAR_MCLOGO:
-	    case CHAR_TEXTONLY_START:
-	    case CHAR_TEXTONLY_END:
-		break;
-	    case CHAR_XONLY_START:
-		while (*p && *p != CHAR_NODE_END && *p != CHAR_XONLY_END)
-		    p++;
-		if (*p == CHAR_NODE_END || !*p)
-		    p--;
-		break;
 	    default:
 		if (!painting)
 		    continue;
@@ -441,15 +413,11 @@ static void show (Dlg_head *h, char *paint_start)
 		    if (c == ' ' || c == '.')
 			addch (c);
 		    else
-#ifndef OS2_NT
 #ifndef HAVE_SLANG
 			addch (acs_map [c]);
 #else
 			SLsmg_draw_object (h->y + line + 2, h->x + col + 2, c);
 #endif
-#else
-			addch (acs2pc (c));
-#endif /* OS2_NT */
 		} else
 		    addch (c);
 		col++;
@@ -474,24 +442,27 @@ static void show (Dlg_head *h, char *paint_start)
 	dlg_move (h, active_line, active_col);
 }
 
-static int help_event (Gpm_Event *event, Widget *w)
+static int
+help_event (Gpm_Event *event, void *vp)
 {
+    Widget *w = vp;
     Link_Area *current_area;
 
     if (! (event->type & GPM_UP))
 	return 0;
 
     /* The event is relative to the dialog window, adjust it: */
+    event->x -= 2;
     event->y -= 2;
-    
+
     if (event->buttons & GPM_B_RIGHT){
-	currentpoint = startpoint = history [history_ptr].page;
+	currentpoint = history [history_ptr].page;
 	selected_item = history [history_ptr].link;
 	history_ptr--;
 	if (history_ptr < 0)
 	    history_ptr = HISTORY_SIZE-1;
 	
-	help_callback (w->parent, 0, DLG_DRAW);
+	help_callback (w->parent, DLG_DRAW, 0);
 	return 0;
     }
 
@@ -523,7 +494,7 @@ static int help_event (Gpm_Event *event, Widget *w)
 	history_ptr = (history_ptr+1) % HISTORY_SIZE;
 	history [history_ptr].page = currentpoint;
 	history [history_ptr].link = current_area->link_name;
-	currentpoint = startpoint = help_follow_link (currentpoint, current_area->link_name);
+	currentpoint = help_follow_link (currentpoint, current_area->link_name);
 	selected_item = NULL;
     } else{
 	if (event->y < 0)
@@ -537,84 +508,118 @@ static int help_event (Gpm_Event *event, Widget *w)
     }
 
     /* Show the new node */
-    help_callback (w->parent, 0, DLG_DRAW);
+    help_callback (w->parent, DLG_DRAW, 0);
 
     return 0;
 }
 
 /* show help */
 static void
-help_help_cmd (Dlg_head *h)
+help_help_cmd (void *vp)
 {
+    Dlg_head *h = vp;
+    const char *p;
+
     history_ptr = (history_ptr+1) % HISTORY_SIZE;
     history [history_ptr].page = currentpoint;
     history [history_ptr].link = selected_item;
-    currentpoint = startpoint = search_string (data, "[How to use help]") + 1;
+
+    p = search_string(data, "[How to use help]");
+    if (p == NULL)
+	return;
+
+    currentpoint = p + 1; /* Skip the newline following the start of the node */
     selected_item = NULL;
-    help_callback (h, 0, DLG_DRAW);
+    help_callback (h, DLG_DRAW, 0);
 }
 
 static void
-help_index_cmd (Dlg_head *h)
+help_index_cmd (void *vp)
 {
-    char *new_item;
+    Dlg_head *h = vp;
+    const char *new_item;
 
-    history_ptr = (history_ptr+1) % HISTORY_SIZE;
-    history [history_ptr].page = currentpoint;
-    history [history_ptr].link = selected_item;
-    currentpoint = startpoint = search_string (data, "[Help]") + 1;
+    if (!(new_item = search_string (data, "[Contents]"))) {
+	message (1, MSG_ERROR, _(" Cannot find node %s in help file "),
+		 "[Contents]");
+	return;
+    }
 
-    if (!(new_item = search_string (data, "[Contents]")))
-	message (1, MSG_ERROR, _(" Cannot find node [Contents] in help file "));
-    else
-	currentpoint = startpoint = new_item + 1;
+    history_ptr = (history_ptr + 1) % HISTORY_SIZE;
+    history[history_ptr].page = currentpoint;
+    history[history_ptr].link = selected_item;
+
+    currentpoint = new_item + 1; /* Skip the newline following the start of the node */
     selected_item = NULL;
-    help_callback (h, 0, DLG_DRAW);
+    help_callback (h, DLG_DRAW, 0);
 }
 
-static void quit_cmd (void *x)
+static void help_quit_cmd (void *vp)
 {
-    dlg_stop ((Dlg_head *)x);
+    dlg_stop ((Dlg_head *) vp);
 }
 
-static void prev_node_cmd (Dlg_head *h)
+static void prev_node_cmd (void *vp)
 {
-    currentpoint = startpoint = history [history_ptr].page;
+    Dlg_head *h = vp;
+    currentpoint = history [history_ptr].page;
     selected_item = history [history_ptr].link;
     history_ptr--;
     if (history_ptr < 0)
 	history_ptr = HISTORY_SIZE-1;
     
-    help_callback (h, 0, DLG_DRAW);
+    help_callback (h, DLG_DRAW, 0);
 }
 
-static int md_callback (Dlg_head *h, Widget *w, int msg, int par)
+static cb_ret_t
+md_callback (Widget *w, widget_msg_t msg, int parm)
 {
-    return default_proc (h, msg, par);
+    (void) w;
+    return default_proc (msg, parm);
 }
 
-static Widget *mousedispatch_new (int y, int x, int yl, int xl)
+static Widget *
+mousedispatch_new (int y, int x, int yl, int xl)
 {
     Widget *w = g_new (Widget, 1);
 
-    init_widget (w, y, x, yl, xl,
-		 (callback_fn) md_callback, 0, (mouse_h)  help_event, NULL);
+    init_widget (w, y, x, yl, xl, md_callback, help_event);
 
     return w;
 }
 
-static int help_handle_key (struct Dlg_head *h, int c)
+static void help_cmk_move_backward(void *vp, int lines) {
+    (void) &vp;
+    move_backward(lines);
+}
+static void help_cmk_move_forward(void *vp, int lines) {
+    (void) &vp;
+    move_forward(lines);
+}
+static void help_cmk_moveto_top(void *vp, int lines) {
+    (void) &vp;
+    (void) &lines;
+    move_to_top();
+}
+static void help_cmk_moveto_bottom(void *vp, int lines) {
+    (void) &vp;
+    (void) &lines;
+    move_to_bottom();
+}
+
+static cb_ret_t
+help_handle_key (struct Dlg_head *h, int c)
 {
-    char *new_item;
+    const char *new_item;
 
     if (c != KEY_UP && c != KEY_DOWN &&
-	check_movement_keys (c, 1, help_lines, currentpoint,
-			     (movefn) move_backward2,
-			     (movefn) move_forward2,
-			     (movefn) move_to_top,
-			     (movefn) move_to_bottom))
+	check_movement_keys (c, help_lines, NULL,
+			     help_cmk_move_backward,
+			     help_cmk_move_forward,
+			     help_cmk_moveto_top,
+			     help_cmk_moveto_bottom)) {
 	/* Nothing */;
-    else switch (c){
+    } else switch (c){
     case 'l':
     case KEY_LEFT:
 	prev_node_cmd (h);
@@ -634,22 +639,21 @@ static int help_handle_key (struct Dlg_head *h, int c)
 	    if (history_ptr < 0)
 		history_ptr = HISTORY_SIZE-1;
 	    
-	    currentpoint = startpoint = history [history_ptr].page;
+	    currentpoint = history [history_ptr].page;
 	    selected_item   = history [history_ptr].link;
 #endif
 	} else {
 	    history_ptr = (history_ptr+1) % HISTORY_SIZE;
 	    history [history_ptr].page = currentpoint;
 	    history [history_ptr].link = selected_item;
-	    currentpoint = startpoint = help_follow_link (currentpoint, selected_item) + 1;
+	    currentpoint = help_follow_link (currentpoint, selected_item);
 	}
 	selected_item = NULL;
 	break;
 	
     case KEY_DOWN:
     case '\t':
-	/* select next link */
-	new_item = select_next_link (startpoint, selected_item);
+	new_item = select_next_link (selected_item);
 	if (new_item){
 	    selected_item = new_item;
 	    if (selected_item >= last_shown){
@@ -667,9 +671,9 @@ static int help_handle_key (struct Dlg_head *h, int c)
     case KEY_UP:
     case ALT ('\t'):
 	/* select previous link */
-	new_item = select_prev_link (startpoint, selected_item);
+	new_item = select_prev_link (selected_item);
 	selected_item = new_item;
-	if (selected_item < currentpoint || selected_item >= last_shown){
+	if (selected_item == NULL || selected_item < currentpoint) {
 	    if (c == KEY_UP)
 		move_backward (1);
 	    else{
@@ -687,10 +691,13 @@ static int help_handle_key (struct Dlg_head *h, int c)
 	while (*new_item && *new_item != CHAR_NODE_END)
 	    new_item++;
 	if (*++new_item == '['){
-	    while (*new_item != ']')
-		new_item++;
-	    currentpoint = new_item + 2;
-	    selected_item = NULL;
+	    while (*++new_item) {
+		if (*new_item == ']' && *++new_item && *++new_item) {
+		    currentpoint = new_item;
+		    selected_item = NULL;
+		    break;
+		}
+	    }
 	}
 	break;
 	
@@ -718,31 +725,28 @@ static int help_handle_key (struct Dlg_head *h, int c)
 	break;
 
     default:
-	return 0;
+	return MSG_NOT_HANDLED;
 	    
     }
-    help_callback (h, 0, DLG_DRAW);
-    return 1;
+    help_callback (h, DLG_DRAW, 0);
+    return MSG_HANDLED;
 }
 
-static int help_callback (struct Dlg_head *h, int id, int msg)
+static cb_ret_t
+help_callback (struct Dlg_head *h, dlg_msg_t msg, int parm)
 {
-    switch (msg){
+    switch (msg) {
     case DLG_DRAW:
-	attrset (HELP_NORMAL_COLOR);
-	dlg_erase (h);
-	draw_box (h, 1, 1, help_lines+2, HELP_WINDOW_WIDTH+2);
-	attrset (COLOR_HOT_NORMAL);
-	dlg_move (h, 1, (HELP_WINDOW_WIDTH - 1) / 2);
-	addstr (_(" Help "));
-	attrset (HELP_NORMAL_COLOR);
-	show (h, currentpoint);
-	break;
+	common_dialog_repaint (h);
+	help_show (h, currentpoint);
+	return MSG_HANDLED;
 
     case DLG_KEY:
-	return help_handle_key (h, id);
+	return help_handle_key (h, parm);
+
+    default:
+	return default_dlg_callback (h, msg, parm);
     }
-    return 0;
 }
 
 static void
@@ -753,20 +757,20 @@ interactive_display_finish (void)
 }
 
 void
-interactive_display (char *filename, char *node)
+interactive_display (const char *filename, const char *node)
 {
-    WButtonBar	*help_bar;
-    Widget	*md;
-    char	*hlpfile = filename;
+    WButtonBar *help_bar;
+    Widget *md;
+    char *hlpfile = NULL;
 
     if (filename)
 	data = load_file (filename);
     else
 	data = load_mc_home_file ("mc.hlp", &hlpfile);
 
-    if (data == NULL){
-	message (1, MSG_ERROR, _(" Cannot open file %s \n %s "),
-		 hlpfile, unix_error_string (errno));
+    if (data == NULL) {
+	message (1, MSG_ERROR, _(" Cannot open file %s \n %s "), filename ? filename : hlpfile,
+		 unix_error_string (errno));
     }
 
     if (!filename)
@@ -775,55 +779,58 @@ interactive_display (char *filename, char *node)
     if (!data)
 	return;
 
-    if (!(main_node = search_string (data, node))){
-	message (1, MSG_ERROR, _(" Cannot find node %s in help file "), node);
-	interactive_display_finish ();
-	return;
+    if (!node || !*node)
+	node = "[main]";
+
+    if (!(main_node = search_string (data, node))) {
+	message (1, MSG_ERROR, _(" Cannot find node %s in help file "),
+		 node);
+
+	/* Fallback to [main], return if it also cannot be found */
+	main_node = search_string (data, "[main]");
+	if (!main_node) {
+	    interactive_display_finish ();
+	    return;
+	}
     }
 
-    if (help_lines > LINES - 4)
-	help_lines = LINES - 4;
+    help_lines = min (LINES - 4, max (2 * LINES / 3, 18));
 
-    whelp = create_dlg (0, 0, help_lines+4, HELP_WINDOW_WIDTH+4, dialog_colors,
-			help_callback, "[Help]", "help", DLG_TRYUP|DLG_CENTER);
-
-    /* allow us to process the tab key */
-    whelp->raw = 1;
+    whelp =
+	create_dlg (0, 0, help_lines + 4, HELP_WINDOW_WIDTH + 4,
+		    dialog_colors, help_callback, "[Help]", _("Help"),
+		    DLG_TRYUP | DLG_CENTER | DLG_WANT_TAB);
 
     selected_item = search_string_node (main_node, STRING_LINK_START) - 1;
-    currentpoint = startpoint = main_node + 1;
+    currentpoint = main_node + 1; /* Skip the newline following the start of the node */
 
-    for (history_ptr = HISTORY_SIZE; history_ptr;){
+    for (history_ptr = HISTORY_SIZE; history_ptr;) {
 	history_ptr--;
-	history [history_ptr].page = currentpoint;
-	history [history_ptr].link = selected_item;
+	history[history_ptr].page = currentpoint;
+	history[history_ptr].link = selected_item;
     }
 
-    help_bar = buttonbar_new (keybar_visible);
-    help_bar->widget.y -= whelp->y;
-    help_bar->widget.x -= whelp->x;
-    
-    md       = mousedispatch_new (1, 1, help_lines, HELP_WINDOW_WIDTH-2);
-    
-    add_widget (whelp, help_bar);
-    add_widget (whelp, md);
+    help_bar = buttonbar_new (1);
+    ((Widget *) help_bar)->y -= whelp->y;
+    ((Widget *) help_bar)->x -= whelp->x;
 
-    define_label_data (whelp, (Widget *)NULL, 1, _("Help"),
-		       (buttonbarfn) help_help_cmd, whelp);
-    define_label_data (whelp, (Widget *)NULL, 2, _("Index"),
-		       (buttonbarfn) help_index_cmd,whelp);
-    define_label_data (whelp, (Widget *)NULL, 3, _("Prev"),
-		       (buttonbarfn) prev_node_cmd, whelp);
-    define_label (whelp, (Widget *) NULL, 4, "", 0);
-    define_label (whelp, (Widget *) NULL, 5, "", 0);
-    define_label (whelp, (Widget *) NULL, 6, "", 0);
-    define_label (whelp, (Widget *) NULL, 7, "", 0);
-    define_label (whelp, (Widget *) NULL, 8, "", 0);
-    define_label (whelp, (Widget *) NULL, 9, "", 0);
-    define_label_data (whelp, (Widget *) NULL, 10, _("Quit"), quit_cmd, whelp);
+    md = mousedispatch_new (1, 1, help_lines, HELP_WINDOW_WIDTH - 2);
+
+    add_widget (whelp, md);
+    add_widget (whelp, help_bar);
+
+    buttonbar_set_label_data (whelp, 1, _("Help"), help_help_cmd, whelp);
+    buttonbar_set_label_data (whelp, 2, _("Index"), help_index_cmd, whelp);
+    buttonbar_set_label_data (whelp, 3, _("Prev"), prev_node_cmd, whelp);
+    buttonbar_clear_label (whelp, 4);
+    buttonbar_clear_label (whelp, 5);
+    buttonbar_clear_label (whelp, 6);
+    buttonbar_clear_label (whelp, 7);
+    buttonbar_clear_label (whelp, 8);
+    buttonbar_clear_label (whelp, 9);
+    buttonbar_set_label_data (whelp, 10, _("Quit"), help_quit_cmd, whelp);
 
     run_dlg (whelp);
     interactive_display_finish ();
     destroy_dlg (whelp);
 }
-
