@@ -1,5 +1,6 @@
 /* Panel layout module for the Midnight Commander
-   Copyright (C) 1995 the Free Software Foundation
+   Copyright (C) 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+   2006, 2007 Free Software Foundation, Inc.
    
    Written: 1995 Janne Kukonlehto
             1995 Miguel de Icaza
@@ -20,14 +21,14 @@
  */
 
 #include <config.h>
+
+#include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
-#if HAVE_TERMIOS_H
-#    include <termios.h>
-#endif
      /*
       * If TIOCGWINSZ supported, make it available here, because window-
       * resizing code depends on it...
@@ -35,10 +36,10 @@
 #ifdef HAVE_SYS_IOCTL_H
 #   include <sys/ioctl.h>
 #endif
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
+#ifdef HAVE_TERMIOS_H
+#include <termios.h>
 #endif
-#include <signal.h>
+#include <unistd.h>
 
 #include "global.h"
 #include "tty.h"		/* COLS */
@@ -64,15 +65,11 @@
 #include "info.h"		/* The Info widget */
 #include "view.h"		/* The view widget */
 
-#define WANT_WIDGETS
 #include "setup.h"		/* For save_setup() */
 
 /* Controls the display of the rotating dash on the verbose mode */
 int nice_rotating_dash = 1;
 
-/* If set, then we have to call the layout_change routine from main */
-static int layout_do_change = 0;
-
 /* Set if the panels are split horizontally */
 int horizontal_split = 0;
 
@@ -215,22 +212,24 @@ static void update_split (void)
    
     attrset (COLOR_NORMAL);
     dlg_move (layout_dlg, 6, 6);
-    printw ("%03d", _first_panel_size);
+    tty_printf ("%03d", _first_panel_size);
     dlg_move (layout_dlg, 6, 18);
     if (_horizontal_split)
-	printw ("%03d", height - _first_panel_size);
+	tty_printf ("%03d", height - _first_panel_size);
     else
-	printw ("%03d", COLS - _first_panel_size);
+	tty_printf ("%03d", COLS - _first_panel_size);
 }
 
 static int b2left_cback (int action)
 {
+    (void) action;
+
     if (_equal_split){
 	/* Turn equal split off */
 	_equal_split = 0;
 	check_options [6].widget->state = check_options [6].widget->state & ~C_BOOL;
-	dlg_select_widget (layout_dlg, check_options [6].widget);
-	dlg_select_widget (layout_dlg, bleft_widget);
+	dlg_select_widget (check_options [6].widget);
+	dlg_select_widget (bleft_widget);
     }
     _first_panel_size++;
     return 0;
@@ -238,12 +237,14 @@ static int b2left_cback (int action)
 
 static int b2right_cback (int action)
 {
+    (void) action;
+
     if (_equal_split){
 	/* Turn equal split off */
 	_equal_split = 0;
 	check_options [6].widget->state = check_options [6].widget->state & ~C_BOOL;
-	dlg_select_widget (layout_dlg, check_options [6].widget);
-	dlg_select_widget (layout_dlg, bright_widget);
+	dlg_select_widget (check_options [6].widget);
+	dlg_select_widget (bright_widget);
     }
     _first_panel_size--;
     return 0;
@@ -251,6 +252,8 @@ static int b2right_cback (int action)
 
 static int bplus_cback (int action)
 {
+    (void) action;
+
     if (_output_lines < 99)
 	_output_lines++;
     return 0;
@@ -258,6 +261,8 @@ static int bplus_cback (int action)
 
 static int bminus_cback (int action)
 {
+    (void) action;
+
     if (_output_lines > 0)
 	_output_lines--;
     return 0;
@@ -287,7 +292,7 @@ layout_callback (struct Dlg_head *h, dlg_msg_t msg, int parm)
 		dlg_move (h, 9, 16 + first_width);
 		addstr (output_lines_label);
 		dlg_move (h, 9, 10 + first_width);
-		printw ("%02d", _output_lines);
+		tty_printf ("%02d", _output_lines);
 	    }
 	}
 	return MSG_HANDLED;
@@ -329,7 +334,7 @@ layout_callback (struct Dlg_head *h, dlg_msg_t msg, int parm)
 		old_output_lines = _output_lines;
 		attrset (COLOR_NORMAL);
 		dlg_move (h, 9, 10 + first_width);
-		printw ("%02d", _output_lines);
+		tty_printf ("%02d", _output_lines);
 	    }
 	}
 	return MSG_HANDLED;
@@ -479,7 +484,7 @@ init_layout (void)
     _first_panel_size = first_panel_size;
     _output_lines = output_lines;
     add_widget (layout_dlg, check_options[6].widget);
-    radio_widget = radio_new (3, 6, 2, s_split_direction, 1);
+    radio_widget = radio_new (3, 6, 2, s_split_direction);
     add_widget (layout_dlg, radio_widget);
     radio_widget->sel = horizontal_split;
 }
@@ -488,7 +493,6 @@ static void
 layout_change (void)
 {
     setup_panels ();
-    layout_do_change = 0;
     /* re-init the menu, because perhaps there was a change in the way 
        how the panel are split (horizontal/vertical). */
     done_menu ();
@@ -500,6 +504,7 @@ void layout_cmd (void)
 {
     int result;
     int i;
+    int layout_do_change = 0;
 
     init_layout ();
     run_dlg (layout_dlg);
@@ -547,10 +552,6 @@ static void check_split (void)
 void
 init_curses ()
 {
-#ifndef HAS_ACS_AS_PCCHARS
-    if (force_ugly_line_drawing)
-	SLtt_Has_Alt_Charset = 0;
-#endif
     SLsmg_init_smg ();
     do_enter_ca_mode ();
     init_colors ();
@@ -633,10 +634,6 @@ panel_do_cols (int index)
 	set_panel_formats ((WPanel *) panels [index].widget);
     else {
 	panel_update_cols (panels [index].widget, frame_half);
-
-	/* In case of an Hex Quick View must review line length */
-	if (get_display_type (index) == view_quick)
-	    view_update_bytes_per_line ((WView *) panels [index].widget);
     }
 }
 
@@ -702,8 +699,8 @@ setup_panels (void)
 	widget_set_size (&the_prompt->widget, LINES, COLS, 0, 0);
     }
 
-    widget_set_size (&the_bar->widget, LINES - 1, 0, keybar_visible, COLS);
-    the_bar->visible = keybar_visible;
+    widget_set_size ((Widget *) the_bar, LINES - 1, 0, keybar_visible, COLS);
+    buttonbar_set_visible (the_bar, keybar_visible);
 
     /* Output window */
     if (console_flag && output_lines) {
@@ -725,6 +722,7 @@ setup_panels (void)
 
 void flag_winch (int dummy)
 {
+    (void) dummy;
 #ifndef USE_NCURSES	/* don't do malloc in a signal handler */
     low_level_change_screen_size ();
 #endif
@@ -760,6 +758,7 @@ low_level_change_screen_size (void)
 void
 change_screen_size (void)
 {
+    winch_flag = 0;
 #if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
 #if defined TIOCGWINSZ
 
@@ -792,7 +791,6 @@ change_screen_size (void)
     touchwin (stdscr);
 #endif				/* TIOCGWINSZ */
 #endif				/* defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4 */
-    winch_flag = 0;
 }
 
 static int ok_to_refresh = 1;
@@ -836,7 +834,7 @@ void print_vfs_message (const char *msg, ...)
 
 	move (0, 0);
 	attrset (NORMAL_COLOR);
-	printw ("%-*s", COLS-1, str);
+	tty_printf ("%-*s", COLS-1, str);
 
 	/* Restore cursor position */
 	move(row, col);
@@ -961,7 +959,7 @@ void set_display_type (int num, int type)
 	else
 	    file_name = "";
 	
-	view_init ((WView *) new_widget, 0, file_name, 0);
+	view_load ((WView *) new_widget, 0, file_name, 0);
 	break;
     }
     panels [num].type = type;
@@ -973,7 +971,7 @@ void set_display_type (int num, int type)
     /* We use replace to keep the circular list of the dialog in the */
     /* same state.  Maybe we could just kill it and then replace it  */
     if (midnight_dlg && old_widget){
-	dlg_replace_widget (midnight_dlg, old_widget, panels [num].widget);
+	dlg_replace_widget (old_widget, panels [num].widget);
     }
     if (type == view_listing){
 	if (num == 0)
@@ -1040,9 +1038,9 @@ void swap_panels ()
             current_panel = panel1;
 
         if (dlg_widget_active (panels[0].widget))
-            dlg_select_widget (midnight_dlg, (void *) panels [1].widget);
+            dlg_select_widget (panels [1].widget);
         else if (dlg_widget_active (panels[1].widget))
-            dlg_select_widget (midnight_dlg, (void *) panels [0].widget);
+            dlg_select_widget (panels [0].widget);
     } else {
 	WPanel *tmp_panel;
 	
