@@ -1,5 +1,6 @@
 /* Setup loading/saving.
-   Copyright (C) 1994 Miguel de Icaza
+   Copyright (C) 1994, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+   2006, 2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,10 +17,13 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
-#include <sys/types.h>		/* Needed to include local .h files */
-#include <sys/stat.h>
+
 #include <string.h>
 #include <stdio.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+
+#include <mhl/string.h>
 
 #include "global.h"
 #include "tty.h"
@@ -28,7 +32,6 @@
 #include "main.h"
 #include "tree.h"		/* xtree_mode */
 #include "profile.h"
-#define WANT_WIDGETS
 #include "setup.h"
 #include "mouse.h"		/* To make view.h happy */
 #include "view.h"		/* For the externs */
@@ -51,6 +54,7 @@
 
 #ifdef USE_NETCODE
 #   include "../vfs/ftpfs.h"
+#   include "../vfs/fish.h"
 #endif
 
 #ifdef USE_INTERNAL_EDIT
@@ -138,7 +142,7 @@ static const struct {
 static const struct {
     const char *opt_name;
     int  *opt_addr;
-} options [] = {
+} int_options [] = {
     { "show_backups", &show_backups },
     { "show_dot_files", &show_dot_files },
     { "verbose", &verbose },
@@ -157,6 +161,7 @@ static const struct {
     { "confirm_overwrite", &confirm_overwrite },
     { "confirm_execute", &confirm_execute },
     { "confirm_exit", &confirm_exit },
+    { "confirm_directory_hotlist_delete", &confirm_directory_hotlist_delete },
     { "safe_delete", &safe_delete },
     { "mouse_repeat_rate", &mou_auto_repeat },
     { "double_click_speed", &double_click_speed },
@@ -175,7 +180,6 @@ static const struct {
     { "old_esc_mode", &old_esc_mode },
     { "cd_symlinks", &cd_symlinks },
     { "show_all_if_ambiguous", &show_all_if_ambiguous },
-    { "have_fast_cpu", &have_fast_cpu },
     { "max_dirt_limit", &max_dirt_limit },
     { "torben_fj_mode", &torben_fj_mode },
     { "use_file_to_guess_type", &use_file_to_check_type },
@@ -196,6 +200,7 @@ static const struct {
     { "ftpfs_use_passive_connections", &ftpfs_use_passive_connections },
     { "ftpfs_use_unix_list_options", &ftpfs_use_unix_list_options },
     { "ftpfs_first_cd_then_ls", &ftpfs_first_cd_then_ls },
+    { "fish_directory_timeout", &fish_directory_timeout },
 #endif /* USE_NETCODE */
 #endif /* USE_VFS */
 #ifdef USE_INTERNAL_EDIT
@@ -208,7 +213,6 @@ static const struct {
     { "editor_fake_half_tabs", &option_fake_half_tabs },
     { "editor_option_save_mode", &option_save_mode },
     { "editor_option_save_position", &option_save_position },
-    { "editor_option_backup_ext_int", &option_backup_ext_int },
     { "editor_option_auto_para_formatting", &option_auto_para_formatting },
     { "editor_option_typewriter_wrap", &option_typewriter_wrap },
     { "editor_edit_confirm_save", &edit_confirm_save },
@@ -217,9 +221,21 @@ static const struct {
 
     { "nice_rotating_dash", &nice_rotating_dash },
     { "horizontal_split",   &horizontal_split },
+    { "mcview_remember_file_position", &mcview_remember_file_position },
     { 0, 0 }
 };
 
+static const struct {
+    const char *opt_name;
+    char **opt_addr;
+    const char *opt_defval;
+} str_options [] = {
+#ifdef USE_INTERNAL_EDIT
+    { "editor_backup_extension", &option_backup_ext, "~" },
+#endif
+    { NULL, NULL }
+};
+
 void
 panel_save_setup (struct WPanel *panel, const char *section)
 {
@@ -230,6 +246,8 @@ panel_save_setup (struct WPanel *panel, const char *section)
     save_string (section, "reverse", buffer, profile_name);
     g_snprintf (buffer, sizeof (buffer), "%d", panel->case_sensitive);
     save_string (section, "case_sensitive", buffer, profile_name);
+    g_snprintf (buffer, sizeof (buffer), "%d", panel->exec_first);
+    save_string (section, "exec_first", buffer, profile_name);
     for (i = 0; sort_names [i].key; i++)
 	if (sort_names [i].sort_type == (sortfn *) panel->sort_type){
 	    save_string (section, "sort_order",
@@ -264,7 +282,7 @@ save_layout (void)
     int  i;
     char buffer [BUF_TINY];
 
-    profile = concat_dir_and_file (home_dir, PROFILE_NAME);
+    profile = mhl_str_dir_plus_file (home_dir, PROFILE_NAME);
 
     /* Save integer options */
     for (i = 0; layout [i].opt_name; i++){
@@ -281,11 +299,16 @@ save_configure (void)
     char *profile;
     int  i;
 
-    profile = concat_dir_and_file (home_dir, PROFILE_NAME);
+    profile = mhl_str_dir_plus_file (home_dir, PROFILE_NAME);
 
     /* Save integer options */
-    for (i = 0; options [i].opt_name; i++)
-	set_int (profile, options [i].opt_name, *options [i].opt_addr);
+    for (i = 0; int_options[i].opt_name; i++)
+	set_int (profile, int_options[i].opt_name, *int_options[i].opt_addr);
+
+    /* Save string options */
+    for (i = 0; str_options[i].opt_name != NULL; i++)
+	set_config_string (profile, str_options[i].opt_name,
+	    *str_options[i].opt_addr);
 
     g_free (profile);
 }
@@ -324,7 +347,7 @@ save_setup (void)
     char *profile;
 
     saving_setup = 1;
-    profile = concat_dir_and_file (home_dir, PROFILE_NAME);
+    profile = mhl_str_dir_plus_file (home_dir, PROFILE_NAME);
 
     save_configure ();
 
@@ -366,6 +389,7 @@ panel_load_setup (WPanel *panel, const char *section)
 
     panel->reverse = load_int (section, "reverse", 0);
     panel->case_sensitive = load_int (section, "case_sensitive", OS_SORT_CASE_SENSITIVE_DEFAULT);
+    panel->exec_first = load_int (section, "exec_first", 0);
 
     /* Load sort order */
     load_string (section, "sort_order", "name", buffer, sizeof (buffer));
@@ -458,9 +482,9 @@ setup_init (void)
     if (profile_name)
 	    return profile_name;
 
-    profile = concat_dir_and_file (home_dir, PROFILE_NAME);
+    profile = mhl_str_dir_plus_file (home_dir, PROFILE_NAME);
     if (!exist_file (profile)){
-	inifile = concat_dir_and_file (mc_home, "mc.ini");
+	inifile = mhl_str_dir_plus_file (mc_home, "mc.ini");
 	if (exist_file (inifile)){
 	    g_free (profile);
 	    profile = inifile;
@@ -483,12 +507,17 @@ load_setup (void)
 
     /* mc.lib is common for all users, but has priority lower than
        ~/.mc/ini.  FIXME: it's only used for keys and treestore now */
-    global_profile_name = concat_dir_and_file (mc_home, "mc.lib");
+    global_profile_name = mhl_str_dir_plus_file (mc_home, "mc.lib");
 
     /* Load integer boolean options */
-    for (i = 0; options [i].opt_name; i++)
-	*options [i].opt_addr =
-	    get_int (profile, options [i].opt_name, *options [i].opt_addr);
+    for (i = 0; int_options[i].opt_name; i++)
+	*int_options[i].opt_addr =
+	    get_int (profile, int_options[i].opt_name, *int_options[i].opt_addr);
+
+    /* Load string options */
+    for (i = 0; str_options[i].opt_name != NULL; i++)
+	*str_options[i].opt_addr = get_config_string (profile,
+	    str_options[i].opt_name, str_options[i].opt_defval);
 
     load_layout (profile);
 
