@@ -1,5 +1,6 @@
 /* Directory tree browser for the Midnight Commander
-   Copyright (C) 1994, 1995, 1996, 1997 The Free Software Foundation
+   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
+   2003, 2004, 2005, 2007 Free Software Foundation, Inc.
 
    Written: 1994, 1996 Janne Kukonlehto
             1997 Norbert Warmuth
@@ -27,9 +28,10 @@
 
    */
 #include <config.h>
-#include <string.h>
+
 #include <errno.h>
 #include <stdio.h>
+#include <string.h>
 
 #include "global.h"
 #include "tty.h"
@@ -47,11 +49,14 @@
 #include "tree.h"
 #include "treestore.h"
 #include "cmd.h"
+#include "history.h"
+
+#define tlines(t) (t->is_panel ? t->widget.lines-2 - (show_mini_info ? 2 : 0) : t->widget.lines)
 
 extern int command_prompt;
 
 /* Use the color of the parent widget for the unselected entries */
-#define TREE_NORMALC NORMALC
+#define TREE_NORMALC(h) (DLG_NORMALC (h))
 
 /* Specifies the display mode: 1d or 2d */
 static int tree_navigation_flag;
@@ -71,7 +76,7 @@ struct WTree {
 
 /* Forwards */
 static void save_tree (WTree *tree);
-static void tree_rescan_cmd (WTree *tree);
+static void tree_rescan_cmd (WTree *);
 
 static tree_entry *back_ptr (tree_entry *ptr, int *count)
 {
@@ -112,6 +117,7 @@ remove_callback (tree_entry *entry, void *data)
 
 static void tree_remove_entry (WTree *tree, char *name)
 {
+    (void) tree;
     tree_store_remove_entry (name);
 }
 
@@ -139,12 +145,12 @@ static void save_tree (WTree *tree)
 {
     int error;
 
+    (void) tree;
     error = tree_store_save ();
 
     if (error){
 	fprintf (stderr, _("Cannot open the %s file for writing:\n%s\n"), MC_TREE,
 		 unix_error_string (error));
-	return;
     }
 }
 
@@ -167,13 +173,13 @@ static void tree_show_mini_info (WTree *tree, int tree_lines, int tree_cols)
 
     if (tree->searching){
 	/* Show search string */
-	attrset (TREE_NORMALC);
-	attrset (FOCUSC);
+	attrset (TREE_NORMALC (h));
+	attrset (DLG_FOCUSC (h));
 	addch (PATH_SEP);
 
 	addstr ((char *) name_trunc (tree->search_buffer, tree_cols-2));
 	addch (' ');
-	attrset (FOCUSC);
+	attrset (DLG_FOCUSC (h));
     } else {
 	/* Show full name of selected directory */
 	addstr ((char *) name_trunc (tree->selected_ptr->name, tree_cols));
@@ -193,7 +199,7 @@ static void show_tree (WTree *tree)
     tree_lines = tlines (tree);
     tree_cols  = tree->widget.cols;
 
-    attrset (TREE_NORMALC);
+    attrset (TREE_NORMALC (h));
     widget_move ((Widget*)tree, y, x);
     if (tree->is_panel){
 	tree_cols  -= 2;
@@ -302,7 +308,7 @@ static void show_tree (WTree *tree)
 	addch (' ');
 
 	/* Return to normal color */
-	attrset (TREE_NORMALC);
+	attrset (TREE_NORMALC (h));
 
 	/* Calculate the next value for current */
 	current = current->next;
@@ -458,7 +464,8 @@ sync_tree (const char *path)
 }
 
 /* Handle mouse click */
-static void tree_event (WTree *tree, int y)
+static void
+tree_event (WTree *tree, int y)
 {
     if (tree->tree_shown [y]){
 	tree->selected_ptr = tree->tree_shown [y];
@@ -478,8 +485,11 @@ static void maybe_chdir (WTree *tree)
 }
 
 /* Mouse callback */
-static int event_callback (Gpm_Event *event, WTree *tree)
+static int
+event_callback (Gpm_Event *event, void *data)
 {
+    WTree *tree = data;
+
     if (!(event->type & GPM_UP))
 	return MOU_NORMAL;
 
@@ -557,7 +567,8 @@ static void tree_do_search (WTree *tree, int key)
     maybe_chdir (tree);
 }
 
-static void tree_rescan_cmd (WTree *tree)
+static void
+tree_rescan_cmd (WTree *tree)
 {
     char old_dir [MC_MAXPATHLEN];
 
@@ -569,11 +580,12 @@ static void tree_rescan_cmd (WTree *tree)
     mc_chdir (old_dir);
 }
 
-static int tree_forget_cmd (WTree *tree)
+static void
+tree_forget_cmd (void *data)
 {
+    WTree *tree = data;
     if (tree->selected_ptr)
 	tree_remove_entry (tree, tree->selected_ptr->name);
-    return 1;
 }
 
 static void tree_copy (WTree *tree, const char *default_dest)
@@ -587,7 +599,7 @@ static void tree_copy (WTree *tree, const char *default_dest)
 	return;
     g_snprintf (cmd_buf, sizeof(cmd_buf), _("Copy \"%s\" directory to:"),
 	     name_trunc (tree->selected_ptr->name, 50));
-    dest = input_expand_dialog (_(" Copy "), cmd_buf, default_dest);
+    dest = input_expand_dialog (_(" Copy "), cmd_buf, MC_HISTORY_FM_TREE_COPY, default_dest);
 
     if (!dest)
 	return;
@@ -605,15 +617,17 @@ static void tree_copy (WTree *tree, const char *default_dest)
     g_free (dest);
 }
 
-static void tree_help_cmd (void)
+static void
+tree_help_cmd (void)
 {
     interactive_display (NULL,  "[Directory Tree]");
 }
 
-static int tree_copy_cmd (WTree *tree)
+static void
+tree_copy_cmd (void *data)
 {
+    WTree *tree = data;
     tree_copy (tree, "");
-    return 1;
 }
 
 static void tree_move (WTree *tree, const char *default_dest)
@@ -628,7 +642,7 @@ static void tree_move (WTree *tree, const char *default_dest)
 	return;
     g_snprintf (cmd_buf, sizeof (cmd_buf), _("Move \"%s\" directory to:"),
 	     name_trunc (tree->selected_ptr->name, 50));
-    dest = input_expand_dialog (_(" Move "), cmd_buf, default_dest);
+    dest = input_expand_dialog (_(" Move "), cmd_buf, MC_HISTORY_FM_TREE_MOVE, default_dest);
     if (!dest)
 	return;
     if (!*dest){
@@ -656,31 +670,30 @@ static void tree_move (WTree *tree, const char *default_dest)
     g_free (dest);
 }
 
-static int
-tree_move_cmd (WTree *tree)
+static void
+tree_move_cmd (void *data)
 {
+    WTree *tree = data;
     tree_move (tree, "");
-    return 1;
 }
 
 #if 0
-static int
+static void
 tree_mkdir_cmd (WTree *tree)
 {
     char old_dir [MC_MAXPATHLEN];
 
     if (!tree->selected_ptr)
-	return 0;
+	return;
     if (!mc_get_current_wd (old_dir, MC_MAXPATHLEN))
-	return 0;
+	return;
     if (chdir (tree->selected_ptr->name))
-	return 0;
+	return;
     /* FIXME
     mkdir_cmd (tree);
     */
     tree_rescan_cmd (tree);
     chdir (old_dir);
-    return 1;
 }
 #endif
 
@@ -719,8 +732,10 @@ tree_rmdir_cmd (WTree *tree)
 static void set_navig_label (WTree *tree);
 
 static void
-tree_toggle_navig (WTree *tree)
+tree_toggle_navig (void *data)
 {
+    WTree *tree = data;
+    /* FIXME: invalid use of boolean variable */
     tree_navigation_flag = 1 - tree_navigation_flag;
     set_navig_label (tree);
 }
@@ -728,9 +743,9 @@ tree_toggle_navig (WTree *tree)
 static void
 set_navig_label (WTree *tree)
 {
-    define_label_data (tree->widget.parent, 4,
+    buttonbar_set_label_data (tree->widget.parent, 4,
 		       tree_navigation_flag ? _("Static") : _("Dynamc"),
-		       (buttonbarfn) tree_toggle_navig, tree);
+		       tree_toggle_navig, tree);
 }
 
 static void
@@ -903,10 +918,10 @@ tree_key (WTree *tree, int key)
     /* We do not want to use them if we do not need to */
     /* Input line may want to take the motion key event */
     if (key == KEY_LEFT)
-	return move_left (tree);
+	return move_left (tree) ? MSG_HANDLED : MSG_NOT_HANDLED;
 
     if (key == KEY_RIGHT)
-	return move_right (tree);
+	return move_right (tree) ? MSG_HANDLED : MSG_NOT_HANDLED;
 
     if (is_abort_char (key)) {
 	if (tree->is_panel) {
@@ -932,7 +947,7 @@ tree_key (WTree *tree, int key)
 	    tree_do_search (tree, key);
 	    return MSG_HANDLED;
 	}
-	return tree->is_panel;
+	return tree->is_panel ? MSG_HANDLED : MSG_NOT_HANDLED;
     }
 
     return MSG_NOT_HANDLED;
@@ -953,10 +968,24 @@ tree_frame (Dlg_head *h, WTree *tree)
     }
 }
 
+static void
+tree_rescan_command (void *data)
+{
+    WTree *tree = data;
+    tree_rescan_cmd (tree);
+}
+
+static void
+tree_rmdir_command (void *data)
+{
+    WTree *tree = data;
+    tree_rmdir_cmd (tree);
+}
 
 static cb_ret_t
-tree_callback (WTree *tree, widget_msg_t msg, int parm)
+tree_callback (Widget *w, widget_msg_t msg, int parm)
 {
+    WTree *tree = (WTree *) w;
     Dlg_head *h = tree->widget.parent;
 
     switch (msg) {
@@ -970,26 +999,21 @@ tree_callback (WTree *tree, widget_msg_t msg, int parm)
 
     case WIDGET_FOCUS:
 	tree->active = 1;
-	define_label (h, 1, _("Help"), (voidfn) tree_help_cmd);
-	define_label_data (h, 2, _("Rescan"),
-			   (buttonbarfn) tree_rescan_cmd, tree);
-	define_label_data (h, 3, _("Forget"),
-			   (buttonbarfn) tree_forget_cmd, tree);
-	define_label_data (h, 5, _("Copy"), (buttonbarfn) tree_copy_cmd,
-			   tree);
-	define_label_data (h, 6, _("RenMov"), (buttonbarfn) tree_move_cmd,
-			   tree);
+	buttonbar_set_label (h, 1, _("Help"), tree_help_cmd);
+	buttonbar_set_label_data (h, 2, _("Rescan"),
+	    tree_rescan_command, tree);
+	buttonbar_set_label_data (h, 3, _("Forget"), tree_forget_cmd, tree);
+	buttonbar_set_label_data (h, 5, _("Copy"), tree_copy_cmd, tree);
+	buttonbar_set_label_data (h, 6, _("RenMov"), tree_move_cmd, tree);
 #if 0
 	/* FIXME: mkdir is currently defunct */
-	define_label_data (h, 7, _("Mkdir"), (buttonbarfn) tree_mkdir_cmd,
-			   tree);
+	buttonbar_set_label_data (h, 7, _("Mkdir"), tree_mkdir_cmd, tree);
 #else
-	define_label (h, 7, "", 0);
+	buttonbar_clear_label (h, 7);
 #endif
-	define_label_data (h, 8, _("Rmdir"), (buttonbarfn) tree_rmdir_cmd,
-			   tree);
+	buttonbar_set_label_data (h, 8, _("Rmdir"), tree_rmdir_command, tree);
 	set_navig_label (tree);
-	redraw_labels (h);
+	buttonbar_redraw (h);
 
 
 	/* FIXME: Should find a better way of only displaying the
@@ -1020,7 +1044,7 @@ tree_new (int is_panel, int y, int x, int lines, int cols)
     WTree *tree = g_new (WTree, 1);
 
     init_widget (&tree->widget, y, x, lines, cols,
-		 (callback_fn) tree_callback, (mouse_h) event_callback);
+		 tree_callback, event_callback);
     tree->is_panel = is_panel;
     tree->selected_ptr = 0;
 
