@@ -1,5 +1,6 @@
 /* User Menu implementation
-   Copyright (C) 1994 Miguel de Icaza, Janne Kukonlehto
+   Copyright (C) 1994, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+   2006, 2007 Free Software Foundation, Inc.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,10 +17,13 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
-#include <stdio.h>
-#include <string.h>
+
 #include <ctype.h>
 #include <errno.h>
+#include <stdio.h>
+#include <string.h>
+
+#include <mhl/string.h>
 
 #include "global.h"
 #include "tty.h"
@@ -27,11 +31,11 @@
 #include "dir.h"
 #include "panel.h"
 #include "main.h"
-#include "subshell.h" 		/* for subshell_pty */
 #include "user.h"
 #include "layout.h"
 #include "execute.h"
 #include "setup.h"
+#include "history.h"
 
 #include "../edit/edit.h"		/* BLOCK_FILE */
 #include "../edit/edit-widget.h"	/* WEdit */
@@ -234,7 +238,7 @@ expand_format (struct WEdit *edit_widget, char c, int quote)
     case 'k':			/* block file name */
     case 'b':			/* block file name / strip extension */  {
 	    if (edit_widget) {
-		char *file = g_strconcat (home_dir, BLOCK_FILE, (char *) NULL);
+		char *file = g_strconcat (home_dir, PATH_SEP_STR BLOCK_FILE, (char *) NULL);
 		fname = (*quote_func) (file, 0);
 		g_free (file);
 		return fname;
@@ -603,7 +607,7 @@ execute_menu_command (WEdit *edit_widget, const char *commands)
 	    if (*commands == '}'){
 		char *tmp;
 		*parameter = 0;
-		parameter = input_dialog (_(" Parameter "), prompt, "");
+		parameter = input_dialog (_(" Parameter "), prompt, MC_HISTORY_FM_MENU_EXEC_PARAM, "");
 		if (!parameter || !*parameter){
 		    /* User canceled */
 		    fclose (cmd_file);
@@ -625,9 +629,9 @@ execute_menu_command (WEdit *edit_widget, const char *commands)
 	    }
 	} else if (expand_prefix_found){
 	    expand_prefix_found = 0;
-	    if (isdigit ((unsigned)*commands)) {
+	    if (isdigit ((unsigned char) *commands)) {
 		do_quote = atoi (commands);
-		while (isdigit ((unsigned)*commands))
+		while (isdigit ((unsigned char) *commands))
 		    commands++;
 	    }
 	    if (*commands == '{')
@@ -655,9 +659,13 @@ execute_menu_command (WEdit *edit_widget, const char *commands)
     chmod (file_name, S_IRWXU);
     if (run_view) {
 	run_view = 0;
-	view (file_name, 0, &run_view, 0);
+	mc_internal_viewer (file_name, NULL, &run_view, 0);
     } else {
-	shell_execute (file_name, EXECUTE_HIDE);
+	/* execute the command indirectly to allow execution even
+	 * on no-exec filesystems. */
+	char *cmd = g_strconcat("/bin/sh ", file_name, (char *)NULL);
+	shell_execute (cmd, EXECUTE_HIDE);
+	g_free(cmd);
     }
     unlink (file_name);
     g_free (file_name);
@@ -715,11 +723,11 @@ user_menu_cmd (struct WEdit *edit_widget)
     menu = g_strdup (edit_widget ? CEDIT_LOCAL_MENU : MC_LOCAL_MENU);
     if (!exist_file (menu) || !menu_file_own (menu)){
 	g_free (menu);
-        menu = concat_dir_and_file \
+        menu = mhl_str_dir_plus_file \
                             (home_dir, edit_widget ? CEDIT_HOME_MENU : MC_HOME_MENU);
 	if (!exist_file (menu)){
 	    g_free (menu);
-	    menu = concat_dir_and_file \
+	    menu = mhl_str_dir_plus_file \
                         (mc_home, edit_widget ? CEDIT_GLOBAL_MENU : MC_GLOBAL_MENU);
 	}
     }
