@@ -1,14 +1,22 @@
-#ifndef __UTIL_H
-#define __UTIL_H
-
+#ifndef MC_UTIL_H
+#define MC_UTIL_H
 
 #include <sys/types.h>
 
+/* Returns its argument as a "modifiable" string. This function is
+ * intended to pass strings to legacy libraries that don't know yet
+ * about the "const" modifier. The return value of this function
+ * MUST NOT be modified. */
+extern char *str_unconst (const char *);
+
 /* String managing functions */
 
+extern const char *cstrcasestr (const char *haystack, const char *needle);
+extern const char *cstrstr (const char *haystack, const char *needle);
+
 void str_replace(char *s, char from, char to);
 int  is_printable (int c);
-int  msglen (const char *text, int *lines);
+void msglen (const char *text, /*@out@*/ int *lines, /*@out@*/ int *columns);
 
 /* Copy from s to d, and trim the beginning if necessary, and prepend
  * "..." in this case.  The destination string can have at most len
@@ -58,7 +66,6 @@ char *strip_password (char *path, int has_prefix);
 const char *strip_home_and_password (const char *dir);
 
 const char *extension (const char *);
-char *concat_dir_and_file (const char *dir, const char *file);
 const char *unix_error_string (int error_num);
 const char *skip_separators (const char *s);
 const char *skip_numbers (const char *s);
@@ -80,6 +87,8 @@ const char *x_basename (const char *fname);
 /* Profile managing functions */
 int set_int (const char *, const char *, int);
 int get_int (const char *, const char *, int);
+extern char * get_config_string (const char *, const char *, const char *);
+extern void set_config_string (const char *, const char *, const char *);
 
 char *load_file (const char *filename);
 char *load_mc_home_file (const char *filename, char ** allocated_filename);
@@ -165,14 +174,16 @@ int mc_mkstemps(char **pname, const char *prefix, const char *suffix);
 
 char *mc_realpath(const char *path, char resolved_path[]);
 
-enum {
+enum compression_type {
 	COMPRESSION_NONE,
 	COMPRESSION_GZIP,
 	COMPRESSION_BZIP,
 	COMPRESSION_BZIP2
 };
 
-int get_compression_type (int fd);
+/* Looks for ``magic'' bytes at the start of the VFS file to guess the
+ * compression type. Side effect: modifies the file position. */
+enum compression_type get_compression_type (int fd);
 const char *decompress_extension (int type);
 
 /* Hook functions */
@@ -234,15 +245,14 @@ void save_file_position (const char *filename, long line, long column);
 #define ISASCII(c) isascii(c)
 #endif
 
-/* this function allows you to write:
- * char *s = g_strdup("hello, world");
- * s = free_after(g_strconcat(s, s, (char *) NULL), s);
- */
-static inline char *
-free_after (char *result, char *string_to_free)
-{
-	g_free(string_to_free);
-	return result;
-}
+/* usage: str_cmp ("foo", !=, "bar") */
+#define str_cmp(a,rel,b) (strcmp ((a), (b)) rel 0)
 
-#endif				/* !__UTIL_H */
+/* if ch is in [A-Za-z], returns the corresponding control character,
+ * else returns the argument. */
+extern int ascii_alpha_to_cntrl (int ch);
+
+#undef Q_
+const char *Q_ (const char *s);
+
+#endif
