@@ -1,6 +1,7 @@
 /* Widgets for the Midnight Commander
 
-   Copyright (C) 1994, 1995, 1996 the Free Software Foundation
+   Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
+   2004, 2005, 2006, 2007 Free Software Foundation, Inc.
    
    Authors: 1994, 1995 Radek Doulik
             1994, 1995 Miguel de Icaza
@@ -25,12 +26,16 @@
  */
 
 #include <config.h>
-#include <string.h>
-#include <stdio.h>
-#include <errno.h>
-#include <sys/types.h>
-#include <sys/stat.h>
+
+#include <assert.h>
 #include <ctype.h>
+#include <errno.h>
+#include <stdio.h>
+#include <string.h>
+#include <sys/types.h>
+
+#include <mhl/string.h>
+
 #include "global.h"
 #include "tty.h"
 #include "color.h"
@@ -38,19 +43,49 @@
 #include "dialog.h"
 #include "widget.h"
 #include "win.h"
-#include "complete.h"
 #include "key.h"	/* XCTRL and ALT macros  */
 #include "profile.h"	/* for history loading and saving */
 #include "wtools.h"	/* For common_dialog_repaint() */
 #include "main.h"	/* for `slow_terminal' */
 
-static int button_event (Gpm_Event *event, WButton *b);
+#define HISTORY_FILE_NAME ".mc/history"
+
+struct WButtonBar {
+    Widget widget;
+    int    visible;		/* Is it visible? */
+    struct {
+	char   *text;
+	enum { BBFUNC_NONE, BBFUNC_VOID, BBFUNC_PTR } tag;
+	union {
+	    voidfn fn_void;
+	    buttonbarfn fn_ptr;
+	} u;
+	void   *data;
+    } labels [10];
+};
+
+static int button_event (Gpm_Event *event, void *);
 
 int quote = 0;
 
-static cb_ret_t
-button_callback (WButton *b, widget_msg_t msg, int parm)
+static void
+widget_selectcolor (Widget *w, gboolean focused, gboolean hotkey)
 {
+    Dlg_head *h = w->parent;
+
+    attrset (hotkey
+	? (focused
+	    ? DLG_HOT_FOCUSC (h)
+	    : DLG_HOT_NORMALC (h))
+	: (focused
+	    ? DLG_FOCUSC (h)
+	    : DLG_NORMALC (h)));
+}
+
+static cb_ret_t
+button_callback (Widget *w, widget_msg_t msg, int parm)
+{
+    WButton *b = (WButton *) w;
     char buf[BUF_SMALL];
     int stop = 0;
     int off = 0;
@@ -65,17 +100,17 @@ button_callback (WButton *b, widget_msg_t msg, int parm)
 	 * handled by the current widget.
 	 */
 	if (parm == '\n' && h->current == &b->widget) {
-	    button_callback (b, WIDGET_KEY, ' ');
+	    button_callback (w, WIDGET_KEY, ' ');
 	    return MSG_HANDLED;
 	}
 
 	if (parm == '\n' && b->flags == DEFPUSH_BUTTON) {
-	    button_callback (b, WIDGET_KEY, ' ');
+	    button_callback (w, WIDGET_KEY, ' ');
 	    return MSG_HANDLED;
 	}
 
-	if (b->hotkey == parm || toupper (b->hotkey) == parm) {
-	    button_callback (b, WIDGET_KEY, ' ');
+	if (b->hotkey == tolower (parm)) {
+	    button_callback (w, WIDGET_KEY, ' ');
 	    return MSG_HANDLED;
 	}
 
@@ -140,14 +175,14 @@ button_callback (WButton *b, widget_msg_t msg, int parm)
 	    break;
 	}
 
-	attrset ((b->selected) ? FOCUSC : NORMALC);
-	widget_move (&b->widget, 0, 0);
+	widget_selectcolor (w, b->selected, FALSE);
+	widget_move (w, 0, 0);
 
 	addstr (buf);
 
 	if (b->hotpos >= 0) {
-	    attrset ((b->selected) ? HOT_FOCUSC : HOT_NORMALC);
-	    widget_move (&b->widget, 0, b->hotpos + off);
+	    widget_selectcolor (w, b->selected, TRUE);
+	    widget_move (w, 0, b->hotpos + off);
 	    addch ((unsigned char) b->text[b->hotpos]);
 	}
 	return MSG_HANDLED;
@@ -162,13 +197,15 @@ button_callback (WButton *b, widget_msg_t msg, int parm)
 }
 
 static int
-button_event (Gpm_Event *event, WButton *b)
+button_event (Gpm_Event *event, void *data)
 {
+    WButton *b = data;
+
     if (event->type & (GPM_DOWN|GPM_UP)){
     	Dlg_head *h=b->widget.parent;
-	dlg_select_widget (h, b);
+	dlg_select_widget (b);
 	if (event->type & GPM_UP){
-	    button_callback (b, WIDGET_KEY, ' ');
+	    button_callback ((Widget *) data, WIDGET_KEY, ' ');
 	    (*h->callback) (h, DLG_POST_KEY, ' ');
 	    return MOU_NORMAL;
 	}
@@ -208,7 +245,7 @@ button_scan_hotkey (WButton *b)
 
     if (cp != NULL && cp[1] != '\0') {
 	g_strlcpy (cp, cp + 1, strlen (cp));
-	b->hotkey = tolower (*cp);
+	b->hotkey = tolower ((unsigned char) *cp);
 	b->hotpos = cp - b->text;
     }
 }
@@ -220,8 +257,7 @@ button_new (int y, int x, int action, int flags, const char *text,
     WButton *b = g_new (WButton, 1);
 
     init_widget (&b->widget, y, x, 1, button_len (text, flags),
-		 (callback_fn) button_callback,
-		 (mouse_h) button_event);
+		 button_callback, button_event);
     
     b->action = action;
     b->flags  = flags;
@@ -236,6 +272,12 @@ button_new (int y, int x, int action, int flags, const char *text,
     return b;
 }
 
+const char *
+button_get_text (WButton *b)
+{
+    return b->text;
+}
+
 void
 button_set_text (WButton *b, const char *text)
 {
@@ -248,11 +290,12 @@ button_set_text (WButton *b, const char *text)
 
 
 /* Radio button widget */
-static int radio_event (Gpm_Event *event, WRadio *r);
+static int radio_event (Gpm_Event *event, void *);
 
 static cb_ret_t
-radio_callback (WRadio *r, int msg, int parm)
+radio_callback (Widget *w, widget_msg_t msg, int parm)
 {
+    WRadio *r = (WRadio *) w;
     int i;
     Dlg_head *h = r->widget.parent;
 
@@ -265,14 +308,14 @@ radio_callback (WRadio *r, int msg, int parm)
 	    for (i = 0; i < r->count; i++) {
 		cp = strchr (r->texts[i], '&');
 		if (cp != NULL && cp[1] != '\0') {
-		    int c = tolower (cp[1]);
+		    int c = tolower ((unsigned char) cp[1]);
 
 		    if (c != lp)
 			continue;
 		    r->pos = i;
 
 		    /* Take action */
-		    radio_callback (r, WIDGET_KEY, ' ');
+		    radio_callback (w, WIDGET_KEY, ' ');
 		    return MSG_HANDLED;
 		}
 	    }
@@ -284,7 +327,7 @@ radio_callback (WRadio *r, int msg, int parm)
 	case ' ':
 	    r->sel = r->pos;
 	    (*h->callback) (h, DLG_ACTION, 0);
-	    radio_callback (r, WIDGET_FOCUS, ' ');
+	    radio_callback (w, WIDGET_FOCUS, ' ');
 	    return MSG_HANDLED;
 
 	case KEY_UP:
@@ -306,7 +349,7 @@ radio_callback (WRadio *r, int msg, int parm)
 
     case WIDGET_CURSOR:
 	(*h->callback) (h, DLG_ACTION, 0);
-	radio_callback (r, WIDGET_FOCUS, ' ');
+	radio_callback (w, WIDGET_FOCUS, ' ');
 	widget_move (&r->widget, r->pos, 1);
 	return MSG_HANDLED;
 
@@ -314,19 +357,17 @@ radio_callback (WRadio *r, int msg, int parm)
     case WIDGET_FOCUS:
     case WIDGET_DRAW:
 	for (i = 0; i < r->count; i++) {
-	    register const unsigned char *cp;
-	    attrset ((i == r->pos
-		      && msg == WIDGET_FOCUS) ? FOCUSC : NORMALC);
+	    register const char *cp;
+	    const gboolean focused = (i == r->pos && msg == WIDGET_FOCUS);
+	    widget_selectcolor (w, focused, FALSE);
 	    widget_move (&r->widget, i, 0);
 
-	    printw ("(%c) ", (r->sel == i) ? '*' : ' ');
+	    tty_printf ("(%c) ", (r->sel == i) ? '*' : ' ');
 	    for (cp = r->texts[i]; *cp; cp++) {
 		if (*cp == '&') {
-		    attrset ((i == r->pos && msg == WIDGET_FOCUS)
-			     ? HOT_FOCUSC : HOT_NORMALC);
+		    widget_selectcolor (w, focused, TRUE);
 		    addch (*++cp);
-		    attrset ((i == r->pos
-			      && msg == WIDGET_FOCUS) ? FOCUSC : NORMALC);
+		    widget_selectcolor (w, focused, FALSE);
 		} else
 		    addch (*cp);
 	    }
@@ -339,16 +380,19 @@ radio_callback (WRadio *r, int msg, int parm)
 }
 
 static int
-radio_event (Gpm_Event *event, WRadio *r)
+radio_event (Gpm_Event *event, void *data)
 {
+    WRadio *r = data;
+    Widget *w = data;
+
     if (event->type & (GPM_DOWN|GPM_UP)){
     	Dlg_head *h = r->widget.parent;
 	
 	r->pos = event->y - 1;
-	dlg_select_widget (h, r);
+	dlg_select_widget (r);
 	if (event->type & GPM_UP){
-	    radio_callback (r, WIDGET_KEY, ' ');
-	    radio_callback (r, WIDGET_FOCUS, 0);
+	    radio_callback (w, WIDGET_KEY, ' ');
+	    radio_callback (w, WIDGET_FOCUS, 0);
 	    (*h->callback) (h, DLG_POST_KEY, ' ');
 	    return MOU_NORMAL;
 	}
@@ -357,7 +401,7 @@ radio_event (Gpm_Event *event, WRadio *r)
 }
 
 WRadio *
-radio_new (int y, int x, int count, const char **texts, int use_hotkey)
+radio_new (int y, int x, int count, const char **texts)
 {
     WRadio *r = g_new (WRadio, 1);
     int i, max, m;
@@ -370,14 +414,12 @@ radio_new (int y, int x, int count, const char **texts, int use_hotkey)
 	    max = m;
     }
 
-    init_widget (&r->widget, y, x, count, max, (callback_fn) radio_callback,
-		 (mouse_h) radio_event);
+    init_widget (&r->widget, y, x, count, max, radio_callback, radio_event);
     r->state = 1;
     r->pos = 0;
     r->sel = 0;
     r->count = count;
     r->texts = texts;
-    r->upper_letter_is_hotkey = use_hotkey;
     widget_want_hotkey (r->widget, 1);
     
     return r;
@@ -386,11 +428,12 @@ radio_new (int y, int x, int count, const char **texts, int use_hotkey)
 
 /* Checkbutton widget */
 
-static int check_event (Gpm_Event *event, WCheck *b);
+static int check_event (Gpm_Event *event, void *);
 
 static cb_ret_t
-check_callback (WCheck *c, widget_msg_t msg, int parm)
+check_callback (Widget *w, widget_msg_t msg, int parm)
 {
+    WCheck *c = (WCheck *) w;
     Dlg_head *h = c->widget.parent;
 
     switch (msg) {
@@ -398,7 +441,7 @@ check_callback (WCheck *c, widget_msg_t msg, int parm)
 	if (c->hotkey == parm
 	    || (c->hotkey >= 'a' && c->hotkey <= 'z'
 		&& c->hotkey - 32 == parm)) {
-	    check_callback (c, WIDGET_KEY, ' ');	/* make action */
+	    check_callback (w, WIDGET_KEY, ' ');	/* make action */
 	    return MSG_HANDLED;
 	}
 	return MSG_NOT_HANDLED;
@@ -409,7 +452,7 @@ check_callback (WCheck *c, widget_msg_t msg, int parm)
 	c->state ^= C_BOOL;
 	c->state ^= C_CHANGE;
 	(*h->callback) (h, DLG_ACTION, 0);
-	check_callback (c, WIDGET_FOCUS, ' ');
+	check_callback (w, WIDGET_FOCUS, ' ');
 	return MSG_HANDLED;
 
     case WIDGET_CURSOR:
@@ -419,12 +462,12 @@ check_callback (WCheck *c, widget_msg_t msg, int parm)
     case WIDGET_FOCUS:
     case WIDGET_UNFOCUS:
     case WIDGET_DRAW:
-	attrset ((msg == WIDGET_FOCUS) ? FOCUSC : NORMALC);
+	widget_selectcolor (w, msg == WIDGET_FOCUS, FALSE);
 	widget_move (&c->widget, 0, 0);
-	printw ("[%c] %s", (c->state & C_BOOL) ? 'x' : ' ', c->text);
+	tty_printf ("[%c] %s", (c->state & C_BOOL) ? 'x' : ' ', c->text);
 
 	if (c->hotpos >= 0) {
-	    attrset ((msg == WIDGET_FOCUS) ? HOT_FOCUSC : HOT_NORMALC);
+	    widget_selectcolor (w, msg == WIDGET_FOCUS, TRUE);
 	    widget_move (&c->widget, 0, +c->hotpos + 4);
 	    addch ((unsigned char) c->text[c->hotpos]);
 	}
@@ -440,15 +483,18 @@ check_callback (WCheck *c, widget_msg_t msg, int parm)
 }
 
 static int
-check_event (Gpm_Event *event, WCheck *c)
+check_event (Gpm_Event *event, void *data)
 {
+    WCheck *c = data;
+    Widget *w = data;
+
     if (event->type & (GPM_DOWN|GPM_UP)){
     	Dlg_head *h = c->widget.parent;
 	
-	dlg_select_widget (h, c);
+	dlg_select_widget (c);
 	if (event->type & GPM_UP){
-	    check_callback (c, WIDGET_KEY, ' ');
-	    check_callback (c, WIDGET_FOCUS, 0);
+	    check_callback (w, WIDGET_KEY, ' ');
+	    check_callback (w, WIDGET_FOCUS, 0);
 	    (*h->callback) (h, DLG_POST_KEY, ' ');
 	    return MOU_NORMAL;
 	}
@@ -464,8 +510,7 @@ check_new (int y, int x, int state, const char *text)
     char *t;
     
     init_widget (&c->widget, y, x, 1, strlen (text),
-		 (callback_fn)check_callback,
-		 (mouse_h) check_event);
+	check_callback, check_event);
     c->state = state ? C_BOOL : 0;
     c->text = g_strdup (text);
     c->hotkey = 0;
@@ -480,7 +525,7 @@ check_new (int y, int x, int state, const char *text)
 	}
 	s++;
 	if (*s){
-	    c->hotkey = tolower (*s);
+	    c->hotkey = tolower ((unsigned char) *s);
 	    c->hotpos = t - c->text;
 	}
 	*t = *s;
@@ -492,9 +537,10 @@ check_new (int y, int x, int state, const char *text)
 
 /* Label widget */
 
-static int
-label_callback (WLabel *l, int msg, int parm)
+static cb_ret_t
+label_callback (Widget *w, widget_msg_t msg, int parm)
 {
+    WLabel *l = (WLabel *) w;
     Dlg_head *h = l->widget.parent;
 
     switch (msg) {
@@ -516,7 +562,7 @@ label_callback (WLabel *l, int msg, int parm)
 	    if (l->transparent)
 		attrset (DEFAULT_COLOR);
 	    else
-		attrset (NORMALC);
+		attrset (DLG_NORMALC (h));
 	    for (;;) {
 		int xlen;
 
@@ -526,10 +572,10 @@ label_callback (WLabel *l, int msg, int parm)
 		    *q = 0;
 		}
 		widget_move (&l->widget, y, 0);
-		printw ("%s", p);
+		tty_printf ("%s", p);
 		xlen = l->widget.cols - strlen (p);
 		if (xlen > 0)
-		    printw ("%*s", xlen, " ");
+		    tty_printf ("%*s", xlen, " ");
 		if (!q)
 		    break;
 		*q = c;
@@ -569,7 +615,7 @@ label_set_text (WLabel *label, const char *text)
 	label->text = 0;
     
     if (label->widget.parent)
-	label_callback (label, WIDGET_DRAW, 0);
+	label_callback ((Widget *) label, WIDGET_DRAW, 0);
 
     if (newcols < label->widget.cols)
         label->widget.cols = newcols;
@@ -588,8 +634,7 @@ label_new (int y, int x, const char *text)
 	width = strlen (text);
 
     l = g_new (WLabel, 1);
-    init_widget (&l->widget, y, x, 1, width,
-		 (callback_fn) label_callback, NULL);
+    init_widget (&l->widget, y, x, 1, width, label_callback, NULL);
     l->text = text ? g_strdup (text) : 0;
     l->auto_adjust_cols = 1;
     l->transparent = 0;
@@ -602,9 +647,10 @@ label_new (int y, int x, const char *text)
 /* Currently width is hardcoded here for text mode */
 #define gauge_len 47
 
-static int
-gauge_callback (WGauge *g, int msg, int parm)
+static cb_ret_t
+gauge_callback (Widget *w, widget_msg_t msg, int parm)
 {
+    WGauge *g = (WGauge *) w;
     Dlg_head *h = g->widget.parent;
 
     if (msg == WIDGET_INIT)
@@ -616,9 +662,9 @@ gauge_callback (WGauge *g, int msg, int parm)
 
     if (msg == WIDGET_DRAW){
 	widget_move (&g->widget, 0, 0);
-	attrset (NORMALC);
+	attrset (DLG_NORMALC (h));
 	if (!g->shown)
-	    printw ("%*s", gauge_len, "");
+	    tty_printf ("%*s", gauge_len, "");
 	else {
 	    int percentage, columns;
 	    long total = g->max, done = g->current;
@@ -637,9 +683,9 @@ gauge_callback (WGauge *g, int msg, int parm)
 	    columns = (2 * (gauge_len - 7) * done / total + 1) / 2;
 	    addch ('[');
 	    attrset (GAUGE_COLOR);
-	    printw ("%*s", (int) columns, "");
-	    attrset (NORMALC);
-	    printw ("%*s] %3d%%", (int)(gauge_len - 7 - columns), "", (int) percentage);
+	    tty_printf ("%*s", (int) columns, "");
+	    attrset (DLG_NORMALC (h));
+	    tty_printf ("%*s] %3d%%", (int)(gauge_len - 7 - columns), "", (int) percentage);
 	}
 	return MSG_HANDLED;
     }
@@ -656,7 +702,7 @@ gauge_set_value (WGauge *g, int max, int current)
 
     g->current = current;
     g->max = max;
-    gauge_callback (g, WIDGET_DRAW, 0);
+    gauge_callback ((Widget *) g, WIDGET_DRAW, 0);
 }
 
 void
@@ -665,7 +711,7 @@ gauge_show (WGauge *g, int shown)
     if (g->shown == shown)
         return;
     g->shown = shown;
-    gauge_callback (g, WIDGET_DRAW, 0);
+    gauge_callback ((Widget *) g, WIDGET_DRAW, 0);
 }
 
 WGauge *
@@ -673,8 +719,7 @@ gauge_new (int y, int x, int shown, int max, int current)
 {
     WGauge *g = g_new (WGauge, 1);
 
-    init_widget (&g->widget, y, x, 1, gauge_len,
-		 (callback_fn) gauge_callback, NULL);
+    init_widget (&g->widget, y, x, 1, gauge_len, gauge_callback, NULL);
     g->shown = shown;
     if (max == 0)
         max = 1; /* I do not like division by zero :) */
@@ -821,7 +866,7 @@ history_get (const char *input_name)
 	return NULL;
     if (!*input_name)
 	return NULL;
-    profile = concat_dir_and_file (home_dir, HISTORY_FILE_NAME);
+    profile = mhl_str_dir_plus_file (home_dir, HISTORY_FILE_NAME);
     for (i = 0;; i++) {
 	char key_name[BUF_TINY];
 	char this_entry[BUF_LARGE];
@@ -859,7 +904,7 @@ history_put (const char *input_name, GList *h)
     if (!num_history_items_recorded)	/* this is how to disable */
 	return;
 
-    profile = concat_dir_and_file (home_dir, HISTORY_FILE_NAME);
+    profile = mhl_str_dir_plus_file (home_dir, HISTORY_FILE_NAME);
 
     if ((i = open (profile, O_CREAT | O_EXCL, S_IRUSR | S_IWUSR)) != -1)
 	close (i);
@@ -913,7 +958,18 @@ i18n_htitle (void)
     return history_title;	
 }
 
-static inline cb_ret_t listbox_fwd (WListbox *l);
+static WLEntry *listbox_select_pos (WListbox *l, WLEntry *base, int
+				    pos);
+
+static inline cb_ret_t
+listbox_fwd (WListbox *l)
+{
+    if (l->current != l->list->prev){
+	listbox_select_entry (l, listbox_select_pos (l, l->list, l->pos+1));
+	return MSG_HANDLED;
+    }
+    return MSG_NOT_HANDLED;
+}
 
 char *
 show_hist (GList *history, int widget_x, int widget_y)
@@ -1025,8 +1081,6 @@ input_destroy (WInput *in)
     g_free (in->history_name);
 }
 
-static char disable_update = 0;
-
 void
 input_disable_update (WInput *in)
 {
@@ -1163,13 +1217,15 @@ forward_char (WInput *in)
 }
 
 static void
-forward_word (WInput *in)
+forward_word (WInput * in)
 {
-    unsigned char *p = in->buffer+in->point;
+    char *p = in->buffer + in->point;
 
-    while (*p && (isspace (*p) || ispunct (*p)))
+    while (*p
+	   && (isspace ((unsigned char) *p)
+	       || ispunct ((unsigned char) *p)))
 	p++;
-    while (*p && isalnum (*p))
+    while (*p && isalnum ((unsigned char) *p))
 	p++;
     in->point = p - in->buffer;
 }
@@ -1177,11 +1233,13 @@ forward_word (WInput *in)
 static void
 backward_word (WInput *in)
 {
-    unsigned char *p = in->buffer+in->point;
+    char *p = in->buffer + in->point;
 
-    while (p-1 > in->buffer-1 && (isspace (*(p-1)) || ispunct (*(p-1))))
+    while (p - 1 > in->buffer - 1 && (isspace ((unsigned char) *(p - 1))
+				      || ispunct ((unsigned char)
+						  *(p - 1))))
 	p--;
-    while (p-1 > in->buffer-1 && isalnum (*(p-1)))
+    while (p - 1 > in->buffer - 1 && isalnum ((unsigned char) *(p - 1)))
 	p--;
     in->point = p - in->buffer;
 }
@@ -1444,7 +1502,9 @@ int
 is_in_input_map (WInput *in, int c_code)
 {
     int i;
-    
+
+    (void) in;
+
     for (i = 0; input_map [i].fn; i++)
 	if (c_code == input_map [i].key_code) {
 	    if (input_map [i].fn == complete)
@@ -1466,10 +1526,10 @@ port_region_marked_for_delete (WInput *in)
 cb_ret_t
 handle_char (WInput *in, int c_code)
 {
+    cb_ret_t v;
     int    i;
-    int    v;
 
-    v = 0;
+    v = MSG_NOT_HANDLED;
 
     if (quote){
     	free_completions (in);
@@ -1484,7 +1544,7 @@ handle_char (WInput *in, int c_code)
 	    if (input_map [i].fn != complete)
 	    	free_completions (in);
 	    (*input_map [i].fn)(in);
-	    v = 1;
+	    v = MSG_HANDLED;
 	    break;
 	}
     }
@@ -1497,8 +1557,7 @@ handle_char (WInput *in, int c_code)
     	free_completions (in);
 	v = insert_char (in, c_code);
     }
-    if (!disable_update)
-	update_input (in, 1);
+    update_input (in, 1);
     return v;
 }
 
@@ -1527,15 +1586,16 @@ input_set_point (WInput *in, int pos)
 }
 
 cb_ret_t
-input_callback (WInput *in, widget_msg_t msg, int parm)
+input_callback (Widget *w, widget_msg_t msg, int parm)
 {
-    int v;
+    WInput *in = (WInput *) w;
+    cb_ret_t v;
 
     switch (msg) {
     case WIDGET_KEY:
 	if (parm == XCTRL ('q')) {
 	    quote = 1;
-	    v = handle_char (in, mi_getch ());
+	    v = handle_char (in, ascii_alpha_to_cntrl (mi_getch ()));
 	    quote = 0;
 	    return v;
 	}
@@ -1575,10 +1635,12 @@ input_callback (WInput *in, widget_msg_t msg, int parm)
 }
 
 static int
-input_event (Gpm_Event * event, WInput * in)
+input_event (Gpm_Event * event, void *data)
 {
+    WInput *in = data;
+
     if (event->type & (GPM_DOWN | GPM_DRAG)) {
-	dlg_select_widget (in->widget.parent, in);
+	dlg_select_widget (in);
 
 	if (event->x >= in->field_len - HISTORY_BUTTON_WIDTH + 1
 	    && should_show_history_button (in)) {
@@ -1597,13 +1659,12 @@ input_event (Gpm_Event * event, WInput * in)
 
 WInput *
 input_new (int y, int x, int color, int len, const char *def_text,
-	   const char *histname)
+	   const char *histname, INPUT_COMPLETE_FLAGS completion_flags)
 {
     WInput *in = g_new (WInput, 1);
     int initial_buffer_len;
 
-    init_widget (&in->widget, y, x, 1, len, (callback_fn) input_callback,
-		 (mouse_h) input_event);
+    init_widget (&in->widget, y, x, 1, len, input_callback, input_event);
 
     /* history setup */
     in->history = NULL;
@@ -1627,9 +1688,7 @@ input_new (int y, int x, int color, int len, const char *def_text,
     initial_buffer_len = 1 + max ((size_t) len, strlen (def_text));
     in->widget.options |= W_IS_INPUT;
     in->completions = NULL;
-    in->completion_flags =
-	INPUT_COMPLETE_FILENAMES | INPUT_COMPLETE_HOSTNAMES |
-	INPUT_COMPLETE_VARIABLES | INPUT_COMPLETE_USERNAMES;
+    in->completion_flags = completion_flags;
     in->current_max_len = initial_buffer_len;
     in->buffer = g_malloc (initial_buffer_len);
     in->color = color;
@@ -1698,14 +1757,14 @@ listbox_draw (WListbox *l, int focused)
     int i;
     int sel_line;
     Dlg_head *h = l->widget.parent;
-    int normalc = NORMALC; 
+    int normalc = DLG_NORMALC (h);
     int selc;
     const char *text; 
 
     if (focused){
-	selc    = FOCUSC;
+	selc    = DLG_FOCUSC (h);
     } else {
-	selc    = HOT_FOCUSC;
+	selc    = DLG_HOT_FOCUSC (h);
     }
     sel_line = -1;
 
@@ -1726,7 +1785,7 @@ listbox_draw (WListbox *l, int focused)
 	    text = e->text;
 	    e = e->next;
 	}
-	printw (" %-*s ", l->width-2, name_trunc (text, l->width-2));
+	tty_printf (" %-*s ", l->width-2, name_trunc (text, l->width-2));
     }
     l->cursor_y = sel_line;
     if (!l->scrollbar)
@@ -1901,16 +1960,6 @@ listbox_back (WListbox *l)
     return MSG_NOT_HANDLED;
 }
 
-static inline cb_ret_t
-listbox_fwd (WListbox *l)
-{
-    if (l->current != l->list->prev){
-	listbox_select_entry (l, listbox_select_pos (l, l->list, l->pos+1));
-	return MSG_HANDLED;
-    }
-    return MSG_NOT_HANDLED;
-}
-
 /* Return MSG_HANDLED if we want a redraw */
 static cb_ret_t
 listbox_key (WListbox *l, int key)
@@ -1952,13 +2001,13 @@ listbox_key (WListbox *l, int key)
     case XCTRL('v'):
 	for (i = 0; i < l->height-1; i++)
 	    j |= listbox_fwd (l);
-	return j > 0;
+	return (j > 0) ? MSG_HANDLED : MSG_NOT_HANDLED;
 	
     case KEY_PPAGE:
     case ALT('v'):
 	for (i = 0; i < l->height-1; i++)
 	    j |= listbox_back (l);
-	return j > 0;
+	return (j > 0) ? MSG_HANDLED : MSG_NOT_HANDLED;
     }
     return MSG_NOT_HANDLED;
 }
@@ -1977,10 +2026,11 @@ listbox_destroy (WListbox *l)
     }
 }
 
-static int
-listbox_callback (WListbox *l, int msg, int parm)
+static cb_ret_t
+listbox_callback (Widget *w, widget_msg_t msg, int parm)
 {
-    int ret_code;
+    WListbox *l = (WListbox *) w;
+    cb_ret_t ret_code;
     WLEntry *e;
     Dlg_head *h = l->widget.parent;
 
@@ -2032,15 +2082,17 @@ listbox_callback (WListbox *l, int msg, int parm)
 }
 
 static int
-listbox_event (Gpm_Event *event, WListbox *l)
+listbox_event (Gpm_Event *event, void *data)
 {
+    WListbox *l = data;
+    Widget *w = data;
     int i;
 
     Dlg_head *h = l->widget.parent;
 
     /* Single click */
     if (event->type & GPM_DOWN)
-	dlg_select_widget (l->widget.parent, l);
+	dlg_select_widget (l);
     if (!l->list)
 	return MOU_NORMAL;
     if (event->type & (GPM_DOWN | GPM_DRAG)) {
@@ -2059,7 +2111,7 @@ listbox_event (Gpm_Event *event, WListbox *l)
 
 	/* We need to refresh ourselves since the dialog manager doesn't */
 	/* know about this event */
-	listbox_callback (l, WIDGET_DRAW, 0);
+	listbox_callback (w, WIDGET_DRAW, 0);
 	mc_refresh ();
 	return MOU_REPEAT;
     }
@@ -2073,7 +2125,7 @@ listbox_event (Gpm_Event *event, WListbox *l)
 	if (event->y < 1 || event->y > l->height)
 	    return MOU_NORMAL;
 
-	dlg_select_widget (l->widget.parent, l);
+	dlg_select_widget (l);
 	listbox_select_entry (l,
 			      listbox_select_pos (l, l->top,
 						  event->y - 1));
@@ -2098,8 +2150,7 @@ listbox_new (int y, int x, int width, int height, lcback callback)
     WListbox *l = g_new (WListbox, 1);
 
     init_widget (&l->widget, y, x, height, width,
-		 (callback_fn) listbox_callback,
-		 (mouse_h) listbox_event);
+		 listbox_callback, listbox_event);
 
     l->list = l->top = l->current = 0;
     l->pos = 0;
@@ -2109,8 +2160,6 @@ listbox_new (int y, int x, int width, int height, lcback callback)
     else
 	l->height = height;
     l->count = 0;
-    l->top = 0;
-    l->current = 0;
     l->cback = callback;
     l->allow_duplicates = 1;
     l->scrollbar = slow_terminal ? 0 : 1;
@@ -2150,6 +2199,22 @@ listbox_append_item (WListbox *l, WLEntry *e, enum append_pos pos)
 	e->next = l->current->next;
 	l->current->next->prev = e;
 	l->current->next = e;
+    } else if (pos == LISTBOX_APPEND_SORTED) {
+	WLEntry *w = l->list;
+
+	while (w->next != l->list && strcmp (e->text, w->text) > 0)
+	    w = w->next;
+	if (w->next == l->list) {
+	    e->prev = w;
+	    e->next = l->list;
+	    w->next = e;
+	    l->list->prev = e;
+	} else {
+	    e->next = w;
+	    e->prev = w->prev;
+	    w->prev->next = e;
+	    w->prev = e;
+	}
     }
     l->count++;
 }
@@ -2216,10 +2281,28 @@ listbox_get_current (WListbox *l, char **string, char **extra)
 	*extra = l->current->data;
 }
 
-
-static int
-buttonbar_callback (WButtonBar *bb, int msg, int parm)
+/* returns TRUE if a function has been called, FALSE otherwise. */
+static gboolean
+buttonbar_call (WButtonBar *bb, int i)
 {
+    switch (bb->labels[i].tag) {
+	case BBFUNC_NONE:
+	    break;
+	case BBFUNC_VOID:
+	    bb->labels[i].u.fn_void ();
+	    return TRUE;
+	case BBFUNC_PTR:
+	    bb->labels[i].u.fn_ptr (bb->labels[i].data);
+	    return TRUE;
+    }
+    return FALSE;
+}
+
+
+static cb_ret_t
+buttonbar_callback (Widget *w, widget_msg_t msg, int parm)
+{
+    WButtonBar *bb = (WButtonBar *) w;
     int i;
 
     switch (msg) {
@@ -2228,10 +2311,8 @@ buttonbar_callback (WButtonBar *bb, int msg, int parm)
 
     case WIDGET_HOTKEY:
 	for (i = 0; i < 10; i++) {
-	    if (parm == KEY_F (i + 1) && bb->labels[i].function) {
-		(*bb->labels[i].function) (bb->labels[i].data);
+	    if (parm == KEY_F (i + 1) && buttonbar_call (bb, i))
 		return MSG_HANDLED;
-	    }
 	}
 	return MSG_NOT_HANDLED;
 
@@ -2240,13 +2321,13 @@ buttonbar_callback (WButtonBar *bb, int msg, int parm)
 	    return MSG_HANDLED;
 	widget_move (&bb->widget, 0, 0);
 	attrset (DEFAULT_COLOR);
-	printw ("%-*s", bb->widget.cols, "");
+	tty_printf ("%-*s", bb->widget.cols, "");
 	for (i = 0; i < COLS / 8 && i < 10; i++) {
 	    widget_move (&bb->widget, 0, i * 8);
 	    attrset (DEFAULT_COLOR);
-	    printw ("%d", i + 1);
+	    tty_printf ("%d", i + 1);
 	    attrset (SELECTED_COLOR);
-	    printw ("%-*s", ((i + 1) * 8 == COLS ? 5 : 6),
+	    tty_printf ("%-*s", ((i + 1) * 8 == COLS ? 5 : 6),
 		    bb->labels[i].text ? bb->labels[i].text : "");
 	    attrset (DEFAULT_COLOR);
 	}
@@ -2264,8 +2345,9 @@ buttonbar_callback (WButtonBar *bb, int msg, int parm)
 }
 
 static int
-buttonbar_event (Gpm_Event *event, WButtonBar *bb)
+buttonbar_event (Gpm_Event *event, void *data)
 {
+    WButtonBar *bb = data;
     int button;
 
     if (!(event->type & GPM_UP))
@@ -2273,8 +2355,8 @@ buttonbar_event (Gpm_Event *event, WButtonBar *bb)
     if (event->y == 2)
 	return MOU_NORMAL;
     button = event->x / 8;
-    if (button < 10 && bb->labels [button].function)
-	(*bb->labels [button].function)(bb->labels [button].data);
+    if (button < 10)
+	buttonbar_call (bb, button);
     return MOU_NORMAL;
 }
 
@@ -2285,13 +2367,12 @@ buttonbar_new (int visible)
     WButtonBar *bb = g_new (WButtonBar, 1);
 
     init_widget (&bb->widget, LINES-1, 0, 1, COLS,
-		 (callback_fn) buttonbar_callback,
-		 (mouse_h) buttonbar_event);
+		 buttonbar_callback, buttonbar_event);
     
     bb->visible = visible;
     for (i = 0; i < 10; i++){
-	bb->labels [i].text     = 0;
-	bb->labels [i].function = 0;
+	bb->labels[i].text = NULL;
+	bb->labels[i].tag = BBFUNC_NONE;
     }
     widget_want_hotkey (bb->widget, 1);
     widget_want_cursor (bb->widget, 0);
@@ -2313,12 +2394,24 @@ find_buttonbar (Dlg_head *h)
 {
     WButtonBar *bb;
 
-    bb = (WButtonBar *) find_widget_type (h, (callback_fn) buttonbar_callback);
+    bb = (WButtonBar *) find_widget_type (h, buttonbar_callback);
     return bb;
 }
 
 void
-define_label_data (Dlg_head *h, int idx, const char *text, buttonbarfn cback,
+buttonbar_clear_label (Dlg_head *h, int idx)
+{
+    WButtonBar *bb = find_buttonbar (h);
+
+    if (!bb)
+	return;
+
+    set_label_text (bb, idx, "");
+    bb->labels[idx - 1].tag = BBFUNC_NONE;
+}
+
+void
+buttonbar_set_label_data (Dlg_head *h, int idx, const char *text, buttonbarfn cback,
 		   void *data)
 {
     WButtonBar *bb = find_buttonbar (h);
@@ -2326,20 +2419,35 @@ define_label_data (Dlg_head *h, int idx, const char *text, buttonbarfn cback,
     if (!bb)
 	return;
 
+    assert (cback != (buttonbarfn) 0);
     set_label_text (bb, idx, text);
-    bb->labels[idx - 1].function = cback;
+    bb->labels[idx - 1].tag = BBFUNC_PTR;
+    bb->labels[idx - 1].u.fn_ptr = cback;
     bb->labels[idx - 1].data = data;
 }
 
 void
-define_label (Dlg_head *h, int idx, const char *text, void (*cback) (void))
+buttonbar_set_label (Dlg_head *h, int idx, const char *text, voidfn cback)
 {
-    define_label_data (h, idx, text, (buttonbarfn) cback, 0);
+    WButtonBar *bb = find_buttonbar (h);
+
+    if (!bb)
+	return;
+
+    assert (cback != (voidfn) 0);
+    set_label_text (bb, idx, text);
+    bb->labels[idx - 1].tag = BBFUNC_VOID;
+    bb->labels[idx - 1].u.fn_void = cback;
 }
 
-/* Redraw labels of the buttonbar */
 void
-redraw_labels (Dlg_head *h)
+buttonbar_set_visible (WButtonBar *bb, gboolean visible)
+{
+    bb->visible = visible;
+}
+
+void
+buttonbar_redraw (Dlg_head *h)
 {
     WButtonBar *bb = find_buttonbar (h);
 
@@ -2350,8 +2458,10 @@ redraw_labels (Dlg_head *h)
 }
 
 static cb_ret_t
-groupbox_callback (WGroupbox *g, widget_msg_t msg, int parm)
+groupbox_callback (Widget *w, widget_msg_t msg, int parm)
 {
+    WGroupbox *g = (WGroupbox *) w;
+
     switch (msg) {
     case WIDGET_INIT:
 	return MSG_HANDLED;
@@ -2385,8 +2495,7 @@ groupbox_new (int x, int y, int width, int height, const char *title)
 {
     WGroupbox *g = g_new (WGroupbox, 1);
 
-    init_widget (&g->widget, y, x, height, width,
-		 (callback_fn) groupbox_callback, NULL);
+    init_widget (&g->widget, y, x, height, width, groupbox_callback, NULL);
 
     g->widget.options &= ~W_WANT_CURSOR;
     widget_want_hotkey (g->widget, 0);
