@@ -1,5 +1,5 @@
 /* Virtual File System garbage collection code
-   Copyright (C) 1995-2003 The Free Software Foundation
+   Copyright (C) 2003, 2004, 2005, 2007 Free Software Foundation, Inc.
 
    Written by: 1995 Miguel de Icaza
                1995 Jakub Jelinek
@@ -31,6 +31,12 @@
 #include <signal.h>
 #include <ctype.h>		/* is_digit() */
 
+#include <mhl/string.h>
+
+#include "../src/global.h"
+#include "../src/tty.h"		/* enable/disable interrupt key */
+#include "../src/wtools.h"	/* message() */
+#include "../src/main.h"	/* print_vfs_message */
 #include "utilvfs.h"
 #include "vfs-impl.h"
 #include "gc.h"
@@ -118,7 +124,7 @@ vfs_getid (struct vfs_class *vclass, const char *dir)
     vfsid id = NULL;
 
     /* append slash if needed */
-    dir1 = concat_dir_and_file (dir, "");
+    dir1 = mhl_str_dir_plus_file (dir, "");
     if (vclass->getid)
 	id = (*vclass->getid) (vclass, dir1);
 
