@@ -1,6 +1,7 @@
 /* Virtual File System: Midnight Commander file system.
    
-   Copyright (C) 1995, 1996, 1997 The Free Software Foundation
+   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
+   2004, 2005, 2007 Free Software Foundation, Inc.
 
    Written by Miguel de Icaza
               Andrej Borsenkow
@@ -48,8 +49,11 @@
 #endif
 #endif
 
+#include "../src/global.h"
+#include "../src/tty.h"		/* enable/disable interrupt key */
+#include "../src/wtools.h"	/* message() */
+#include "../src/main.h"	/* print_vfs_message */
 #include "utilvfs.h"
-
 #include "vfs.h"
 #include "vfs-impl.h"
 #include "mcfs.h"
@@ -100,6 +104,8 @@ mcfs_fill_names (struct vfs_class *me, fill_names_f func)
     int i;
     char *name;
 
+    (void) me;
+
     for (i = 0; i < MCFS_MAX_CONNECTIONS; i++) {
 	if (mcfs_connections[i].host == 0)
 	    continue;
@@ -202,9 +208,9 @@ mcfs_get_remote_port (struct sockaddr_in *sin, int *version)
 	if (port = pmap_getport (sin, RPC_PROGNUM, *version, IPPROTO_TCP))
 	    return port;
 #endif				/* HAVE_PMAP_GETPORT */
-#endif				/* HAVE_PMAP_GETMAPS */
     *version = 1;
     return mcserver_port;
+#endif				/* HAVE_PMAP_GETMAPS */
 }
 
 /* This used to be in utilvfs.c, but as it deals with portmapper, it
@@ -400,7 +406,7 @@ mcfs_get_path (mcfs_connection **mc, const char *path)
 	int f = !strcmp (remote_path, "/~");
 	if (f || !strncmp (remote_path, "/~/", 3)) {
 	    char *s;
-	    s = concat_dir_and_file (mcfs_gethome (*mc),
+	    s = mhl_str_dir_plus_file (mcfs_gethome (*mc),
 				     remote_path + 3 - f);
 	    g_free (remote_path);
 	    remote_path = s;
@@ -522,6 +528,8 @@ mcfs_open (struct vfs_class *me, const char *file, int flags, int mode)
     int result, error_num;
     mcfs_handle *remote_handle;
 
+    (void) me;
+
     if (!(remote_file = mcfs_get_path (&mc, file)))
 	return 0;
 
@@ -616,6 +624,8 @@ mcfs_close (void *data)
 static int
 mcfs_errno (struct vfs_class *me)
 {
+    (void) me;
+
     return my_errno;
 }
 
@@ -642,6 +652,8 @@ mcfs_opendir (struct vfs_class *me, const char *dirname)
     char *remote_dir;
     int result;
 
+    (void) me;
+
     if (!(remote_dir = mcfs_get_path (&mc, dirname)))
 	return 0;
 
@@ -881,6 +893,8 @@ mcfs_stat_cmd (int cmd, const char *path, struct stat *buf)
 static int
 mcfs_stat (struct vfs_class *me, const char *path, struct stat *buf)
 {
+    (void) me;
+
     return mcfs_stat_cmd (MC_STAT, path, buf);
 }
 
@@ -890,6 +904,8 @@ mcfs_lstat (struct vfs_class *me, const char *path, struct stat *buf)
     int path_len = strlen (path);
     int entry_len = strlen (mcfs_readdir_data.dent.d_name);
 
+    (void) me;
+
     /* Hack ... */
     if (strcmp (path + path_len - entry_len,
 		mcfs_readdir_data.dent.d_name) == 0 && cached_lstat_info) {
@@ -925,12 +941,16 @@ mcfs_fstat (void *data, struct stat *buf)
 static int
 mcfs_chmod (struct vfs_class *me, const char *path, int mode)
 {
+    (void) me;
+
     return mcfs_rpc_path_int (MC_CHMOD, path, mode);
 }
 
 static int
 mcfs_chown (struct vfs_class *me, const char *path, int owner, int group)
 {
+    (void) me;
+
     return mcfs_rpc_path_int_int (MC_CHOWN, path, owner, group);
 }
 
@@ -941,6 +961,8 @@ mcfs_utime (struct vfs_class *me, const char *path, struct utimbuf *times)
     int status;
     char *file;
 
+    (void) me;
+
     if (!(file = mcfs_get_path (&mc, path)))
 	return -1;
 
@@ -974,6 +996,8 @@ mcfs_readlink (struct vfs_class *me, const char *path, char *buf, size_t size)
     mcfs_connection *mc;
     size_t len;
 
+    (void) me;
+
     if (!(remote_file = mcfs_get_path (&mc, path)))
 	return -1;
 
@@ -1001,18 +1025,24 @@ mcfs_readlink (struct vfs_class *me, const char *path, char *buf, size_t size)
 static int
 mcfs_unlink (struct vfs_class *me, const char *path)
 {
+    (void) me;
+
     return mcfs_rpc_path (MC_UNLINK, path);
 }
 
 static int
 mcfs_symlink (struct vfs_class *me, const char *n1, const char *n2)
 {
+    (void) me;
+
     return mcfs_rpc_two_paths (MC_SYMLINK, n1, n2);
 }
 
 static int
 mcfs_rename (struct vfs_class *me, const char *a, const char *b)
 {
+    (void) me;
+
     return mcfs_rpc_two_paths (MC_RENAME, a, b);
 }
 
@@ -1023,6 +1053,8 @@ mcfs_chdir (struct vfs_class *me, const char *path)
     mcfs_connection *mc;
     int status, error;
 
+    (void) me;
+
     if (!(remote_dir = mcfs_get_path (&mc, path)))
 	return -1;
 
@@ -1056,28 +1088,36 @@ mcfs_lseek (void *data, off_t offset, int whence)
 static int
 mcfs_mknod (struct vfs_class *me, const char *path, int mode, int dev)
 {
+    (void) me;
+
     return mcfs_rpc_path_int_int (MC_MKNOD, path, mode, dev);
 }
 
 static int
 mcfs_mkdir (struct vfs_class *me, const char *path, mode_t mode)
 {
+    (void) me;
+
     return mcfs_rpc_path_int (MC_MKDIR, path, mode);
 }
 
 static int
 mcfs_rmdir (struct vfs_class *me, const char *path)
 {
+    (void) me;
+
     return mcfs_rpc_path (MC_RMDIR, path);
 }
 
 static int
 mcfs_link (struct vfs_class *me, const char *p1, const char *p2)
 {
+    (void) me;
+
     return mcfs_rpc_two_paths (MC_LINK, p1, p2);
 }
 
-/* Gives up on a socket and reopnes the connection, the child own the socket
+/* Gives up on a socket and reopens the connection, the child own the socket
  * now
  */
 static void
@@ -1125,6 +1165,9 @@ mcfs_forget (const char *path)
 static int
 mcfs_setctl (struct vfs_class *me, const char *path, int ctlop, void *arg)
 {
+    (void) me;
+    (void) arg;
+
     switch (ctlop) {
     case VFS_SETCTL_FORGET:
 	mcfs_forget (path);
