@@ -1,6 +1,7 @@
 /* Utilities for VFS modules.
 
-   Copyright (C) 1988, 1992 Free Software Foundation
+   Copyright (C) 1988, 1992, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+   2005, 2006, 2007 Free Software Foundation, Inc.
    Copyright (C) 1995, 1996 Miguel de Icaza
    
    This program is free software; you can redistribute it and/or
@@ -20,8 +21,14 @@
 #include <config.h>
 #include <ctype.h>
 
+#include "../src/global.h"
+#include "../src/tty.h"		/* enable/disable interrupt key */
+#include "../src/wtools.h"	/* message() */
+#include "../src/main.h"	/* print_vfs_message */
 #include "utilvfs.h"
 #include "vfs.h"
+#include "../src/unixcompat.h"
+#include "../src/history.h"
 
 /* Extract the hostname and username from the path */
 /* path is in the form: [user@]hostname:port/remote-dir, e.g.:
@@ -202,7 +209,7 @@ vfs_findgid (const char *gname)
 int
 vfs_mkstemps (char **pname, const char *prefix, const char *basename)
 {
-    const unsigned char *p;
+    const char *p;
     char *suffix, *q;
     int shift;
     int fd;
@@ -224,7 +231,7 @@ vfs_mkstemps (char **pname, const char *prefix, const char *basename)
     /* Protection against unusual characters */
     q = suffix;
     while (*p && (*p != '#')) {
-	if (strchr (".-_@", *p) || isalnum (*p))
+	if (strchr (".-_@", *p) || isalnum ((unsigned char) *p))
 	    *q++ = *p;
 	p++;
     }
@@ -337,15 +344,16 @@ is_month (const char *str, struct tm *tim)
  * NB: It is assumed there are no whitespaces in month.
  */
 static int
-is_localized_month (const unsigned char *month)
+is_localized_month (const char *month)
 {
     int i = 0;
 
     if (!month)
 	return 0;
 
-    while ((i < 3) && *month && !isdigit (*month) && !iscntrl (*month)
-	   && !ispunct (*month)) {
+    while ((i < 3) && *month && !isdigit ((unsigned char) *month)
+	   && !iscntrl ((unsigned char) *month)
+	   && !ispunct ((unsigned char) *month)) {
 	i++;
 	month++;
     }
@@ -401,157 +409,192 @@ is_year (char *str, struct tm *tim)
     return 1;
 }
 
-/*
- * FIXME: this is broken. Consider following entry:
- * -rwx------   1 root     root            1 Aug 31 10:04 2904 1234
- * where "2904 1234" is filename. Well, this code decodes it as year :-(.
- */
-
-int
-vfs_parse_filetype (char c)
+gboolean
+vfs_parse_filetype (const char *s, size_t *ret_skipped, mode_t *ret_type)
 {
-    switch (c) {
-    case 'd':
-	return S_IFDIR;
-    case 'b':
-	return S_IFBLK;
-    case 'c':
-	return S_IFCHR;
-    case 'l':
-	return S_IFLNK;
-    case 's':			/* Socket */
+    mode_t type;
+
+    switch (*s) {
+	case 'd': type = S_IFDIR; break;
+	case 'b': type = S_IFBLK; break;
+	case 'c': type = S_IFCHR; break;
+	case 'l': type = S_IFLNK; break;
 #ifdef S_IFSOCK
-	return S_IFSOCK;
+	case 's': type = S_IFSOCK; break;
 #else
-	/* If not supported, we fall through to IFIFO */
-	return S_IFIFO;
+	case 's': type = S_IFIFO; break;
 #endif
-    case 'D':			/* Solaris door */
-#ifdef S_IFDOOR
-	return S_IFDOOR;
+#ifdef S_IFDOOR			/* Solaris door */
+	case 'D': type = S_IFDOOR; break;
 #else
-	return S_IFIFO;
+	case 'D': type = S_IFIFO; break;
 #endif
-    case 'p':
-	return S_IFIFO;
-    case 'n':			/* Special named files */
-#ifdef S_IFNAM
-	return S_IFNAM;
-#endif /* S_IFNAM */
-    case 'm':			/* Don't know what these are :-) */
-    case '-':
-    case '?':
-	return S_IFREG;
-    default:
-	return -1;
+	case 'p': type = S_IFIFO; break;
+#ifdef S_IFNAM			/* Special named files */
+	case 'n': type = S_IFNAM; break;
+#else
+	case 'n': type = S_IFREG; break;
+#endif
+	case 'm':		/* Don't know what these are :-) */
+	case '-':
+	case '?': type = S_IFREG; break;
+	default: return FALSE;
     }
+
+    *ret_type = type;
+    *ret_skipped = 1;
+    return TRUE;
 }
 
-int
-vfs_parse_filemode (const char *p)
-{				/* converts rw-rw-rw- into 0666 */
-    int res = 0;
-    switch (*(p++)) {
-    case 'r':
-	res |= 0400;
-	break;
-    case '-':
-	break;
-    default:
-	return -1;
+gboolean
+vfs_parse_fileperms (const char *s, size_t *ret_skipped, mode_t *ret_perms)
+{
+    const char *p;
+    mode_t perms;
+
+    p = s;
+    perms = 0;
+
+    switch (*p++) {
+	case '-': break;
+	case 'r': perms |= S_IRUSR; break;
+	default: return FALSE;
     }
-    switch (*(p++)) {
-    case 'w':
-	res |= 0200;
-	break;
-    case '-':
-	break;
-    default:
-	return -1;
+    switch (*p++) {
+	case '-': break;
+	case 'w': perms |= S_IWUSR; break;
+	default: return FALSE;
     }
-    switch (*(p++)) {
-    case 'x':
-	res |= 0100;
-	break;
-    case 's':
-	res |= 0100 | S_ISUID;
-	break;
-    case 'S':
-	res |= S_ISUID;
-	break;
-    case '-':
-	break;
-    default:
-	return -1;
+    switch (*p++) {
+	case '-': break;
+	case 'S': perms |= S_ISUID; break;
+	case 's': perms |= S_IXUSR | S_ISUID; break;
+	case 'x': perms |= S_IXUSR; break;
+	default: return FALSE;
     }
-    switch (*(p++)) {
-    case 'r':
-	res |= 0040;
-	break;
-    case '-':
-	break;
-    default:
-	return -1;
+    switch (*p++) {
+	case '-': break;
+	case 'r': perms |= S_IRGRP; break;
+	default: return FALSE;
     }
-    switch (*(p++)) {
-    case 'w':
-	res |= 0020;
-	break;
-    case '-':
-	break;
-    default:
-	return -1;
+    switch (*p++) {
+	case '-': break;
+	case 'w': perms |= S_IWGRP; break;
+	default: return FALSE;
     }
-    switch (*(p++)) {
-    case 'x':
-	res |= 0010;
-	break;
-    case 's':
-	res |= 0010 | S_ISGID;
-	break;
-    case 'l':			/* Solaris produces these */
-    case 'S':
-	res |= S_ISGID;
-	break;
-    case '-':
-	break;
-    default:
-	return -1;
+    switch (*p++) {
+	case '-': break;
+	case 'S': perms |= S_ISGID; break;
+	case 'l': perms |= S_ISGID; break; /* found on Solaris */
+	case 's': perms |= S_IXGRP | S_ISGID; break;
+	case 'x': perms |= S_IXGRP; break;
+	default: return FALSE;
     }
-    switch (*(p++)) {
-    case 'r':
-	res |= 0004;
-	break;
-    case '-':
-	break;
-    default:
-	return -1;
+    switch (*p++) {
+	case '-': break;
+	case 'r': perms |= S_IROTH; break;
+	default: return FALSE;
     }
-    switch (*(p++)) {
-    case 'w':
-	res |= 0002;
-	break;
-    case '-':
-	break;
-    default:
-	return -1;
+    switch (*p++) {
+	case '-': break;
+	case 'w': perms |= S_IWOTH; break;
+	default: return FALSE;
     }
-    switch (*(p++)) {
-    case 'x':
-	res |= 0001;
-	break;
-    case 't':
-	res |= 0001 | S_ISVTX;
-	break;
-    case 'T':
-	res |= S_ISVTX;
-	break;
-    case '-':
-	break;
-    default:
-	return -1;
+    switch (*p++) {
+	case '-': break;
+	case 'T': perms |= S_ISVTX; break;
+	case 't': perms |= S_IXOTH | S_ISVTX; break;
+	case 'x': perms |= S_IXOTH; break;
+	default: return FALSE;
     }
-    return res;
+    if (*p == '+') {		/* ACLs on Solaris, HP-UX and others */
+	p++;
+    }
+
+    *ret_skipped = p - s;
+    *ret_perms = perms;
+    return TRUE;
+}
+
+gboolean
+vfs_parse_filemode (const char *s, size_t *ret_skipped,
+		    mode_t *ret_mode)
+{
+    const char *p;
+    mode_t type, perms;
+    size_t skipped;
+
+    p = s;
+
+    if (!vfs_parse_filetype (p, &skipped, &type))
+	return FALSE;
+    p += skipped;
+
+    if (!vfs_parse_fileperms (p, &skipped, &perms))
+	return FALSE;
+    p += skipped;
+
+    *ret_skipped = p - s;
+    *ret_mode = type | perms;
+    return TRUE;
+}
+
+gboolean
+vfs_parse_raw_filemode (const char *s, size_t *ret_skipped,
+		    mode_t *ret_mode)
+{
+    const char *p;
+    mode_t remote_type = 0, local_type, perms = 0;
+
+    p = s;
+
+    // isoctal
+    while(*p >= '0' && *p <= '7')
+    {
+	perms *= 010;
+	perms += (*p - '0');
+	++p;
+    }
+
+    if (*p++ != ' ')
+	return FALSE;
+
+    while(*p >= '0' && *p <= '7')
+    {
+	remote_type *= 010;
+	remote_type += (*p - '0');
+	++p;
+    }
+
+    if (*p++ != ' ')
+	return FALSE;
+
+    /* generated with:
+        $ perl -e 'use Fcntl ":mode";
+                   my @modes = (S_IFDIR, S_IFBLK, S_IFCHR, S_IFLNK, S_IFREG);
+                   foreach $t (@modes) { printf ("%o\n", $t); };'
+        TODO: S_IFDOOR, S_IFIFO, S_IFSOCK (if supported by os)
+              (see vfs_parse_filetype)
+     */
+
+    switch (remote_type)
+    {
+	case 020000:
+	    local_type = S_IFCHR; break;
+	case 040000:
+	    local_type = S_IFDIR; break;
+	case 060000:
+	    local_type = S_IFBLK; break;
+	case 0120000:
+	    local_type = S_IFLNK; break;
+	case 0100000:
+	default:// don't know what is it
+	    local_type = S_IFREG; break;
+    }
+
+    *ret_skipped = p - s;
+    *ret_mode = local_type | perms;
+    return TRUE;
 }
 
 /* This function parses from idx in the columns[] array */
@@ -593,14 +636,13 @@ vfs_parse_filedate (int idx, time_t *t)
 	    return 0;		/* No day */
 
     } else {
-	/* We usually expect:
-	   Mon DD hh:mm
-	   Mon DD  YYYY
+	/* We expect:
+	   3 fields max or we'll see oddities with certain file names.
+	   So both year and time is not allowed.
+	   Mon DD hh:mm[:ss]
+	   Mon DD YYYY
 	   But in case of extfs we allow these date formats:
-	   Mon DD YYYY hh:mm
-	   Mon DD hh:mm YYYY
-	   Wek Mon DD hh:mm:ss YYYY
-	   MM-DD-YY hh:mm
+	   MM-DD-YY hh:mm[:ss]
 	   where Mon is Jan-Dec, DD, MM, YY two digit day, month, year,
 	   YYYY four digit year, hh, mm, ss two digit hour, minute or second. */
 
@@ -636,26 +678,12 @@ vfs_parse_filedate (int idx, time_t *t)
 	}
     }
 
-    /* Here we expect to find time and/or year */
-
-    if (is_num (idx)) {
-	if (is_time (columns[idx], &tim)
-	    || (got_year = is_year (columns[idx], &tim))) {
+    /* Here we expect to find time or year */
+    if (is_num (idx) && (is_time (columns[idx], &tim)
+	    || (got_year = is_year (columns[idx], &tim))))
 	    idx++;
-
-	    /* This is a special case for ctime() or Mon DD YYYY hh:mm */
-	    if (is_num (idx) && (columns[idx + 1][0])) {
-		if (got_year) {
-		    if (is_time (columns[idx], &tim))
-			idx++;	/* time also */
-		} else {
-		    if ((got_year = is_year (columns[idx], &tim)))
-			idx++;	/* year also */
-		}
-	    }
-	}			/* only time or date */
-    } else
-	return 0;		/* Nor time or date */
+    else
+	return 0;		/* Neither time nor date */
 
     /*
      * If the date is less than 6 months in the past, it is shown without year
@@ -683,14 +711,15 @@ vfs_parse_ls_lga (const char *p, struct stat *s, char **filename,
     char *p_copy = NULL;
     char *t = NULL;
     const char *line = p;
+    size_t skipped;
 
     if (strncmp (p, "total", 5) == 0)
 	return 0;
 
-    if ((i = vfs_parse_filetype (*(p++))) == -1)
+    if (!vfs_parse_filetype (p, &skipped, &s->st_mode))
 	goto error;
+    p += skipped;
 
-    s->st_mode = i;
     if (*p == ' ')		/* Notwell 4 */
 	p++;
     if (*p == '[') {
@@ -705,14 +734,13 @@ vfs_parse_ls_lga (const char *p, struct stat *s, char **filename,
 	    s->st_mode |= (S_IRUSR | S_IRGRP | S_IROTH | S_IWUSR);
 	p += 9;
     } else {
-	if ((i = vfs_parse_filemode (p)) == -1)
-	    goto error;
-	s->st_mode |= i;
-	p += 9;
+	size_t skipped;
+	mode_t perms;
 
-	/* This is for an extra ACL attribute (HP-UX) */
-	if (*p == '+')
-	    p++;
+	if (!vfs_parse_fileperms (p, &skipped, &perms))
+	    goto error;
+	p += skipped;
+	s->st_mode |= perms;
     }
 
     p_copy = g_strdup (p);
@@ -767,7 +795,7 @@ vfs_parse_ls_lga (const char *p, struct stat *s, char **filename,
 		goto error;
 	}
 #ifdef HAVE_STRUCT_STAT_ST_RDEV
-	s->st_rdev = ((maj & 0xff) << 8) | (min & 0xffff00ff);
+	s->st_rdev = makedev (maj, min);
 #endif
 	s->st_size = 0;
 
@@ -870,6 +898,5 @@ vfs_die (const char *m)
 char *
 vfs_get_password (const char *msg)
 {
-    return input_dialog (msg, _("Password:"), INPUT_PASSWORD);
+    return input_dialog (msg, _("Password:"), MC_HISTORY_VFS_PASSWORD, INPUT_PASSWORD);
 }
-
