@@ -1,11 +1,7 @@
-#ifndef __UTILVFS_H
-#define __UTILVFS_H
+#ifndef MC_VFS_UTILVFS_H
+#define MC_VFS_UTILVFS_H
 
-#include "../src/global.h"
-
-#include "../src/tty.h"		/* enable/disable interrupt key */
-#include "../src/wtools.h"	/* message() */
-#include "../src/main.h"	/* print_vfs_message */
+#include <sys/stat.h>
 
 /* Flags for vfs_split_url() */
 #define URL_ALLOW_ANON 1
@@ -22,10 +18,17 @@ int vfs_mkstemps (char **pname, const char *prefix, const char *basename);
 void vfs_die (const char *msg);
 char *vfs_get_password (const char *msg);
 
+gboolean vfs_parse_filetype (const char *s, size_t *ret_skipped,
+			     mode_t *ret_type);
+gboolean vfs_parse_fileperms (const char *s, size_t *ret_skipped,
+			      mode_t *ret_perms);
+gboolean vfs_parse_filemode (const char *s, size_t *ret_skipped,
+			     mode_t *ret_mode);
+gboolean vfs_parse_raw_filemode (const char *s, size_t *ret_skipped,
+			     mode_t *ret_mode);
+
 int vfs_parse_ls_lga (const char *p, struct stat *s, char **filename,
 		      char **linkname);
-int vfs_parse_filetype (char c);
-int vfs_parse_filemode (const char *p);
 int vfs_parse_filedate (int idx, time_t *t);
 
-#endif				/* !__UTILVFS_H */
+#endif
